@@ -21,7 +21,8 @@ from math import log
 
 from Onboard             import Exceptions
 from Onboard.utils       import hexstring_to_float, brighten, toprettyxml, \
-                                TreeItem, Version, unicode_str
+                                TreeItem, Version, unicode_str, open_utf8, \
+                                XDGDirs
 
 import Onboard.utils as utils
 
@@ -44,7 +45,10 @@ class Theme:
     # onboard 0.98, added shadow keys
     THEME_FORMAT_1_2 = Version(1, 2)
 
-    THEME_FORMAT = THEME_FORMAT_1_2
+    # onboard 0.99, added key_stroke_width
+    THEME_FORMAT_1_3 = Version(1, 3)
+
+    THEME_FORMAT = THEME_FORMAT_1_3
 
     # core theme members
     # name, type, default
@@ -54,6 +58,7 @@ class Theme:
             ["key_style", "s", "flat"],
             ["roundrect_radius", "d", 0.0],
             ["key_size", "d", 100.0],
+            ["key_stroke_width", "d", 100.0],
             ["key_fill_gradient", "d", 0.0],
             ["key_stroke_gradient", "d", 0.0],
             ["key_gradient_direction", "d", 0.0],
@@ -64,13 +69,13 @@ class Theme:
             ]
 
     def __init__(self):
-        self.modified = False
+        self._modified = False
 
-        self.filename = ""
-        self.is_system = False       # True if this a system theme
-        self.system_exists = False   # True if there exists a system
+        self._filename = ""
+        self._is_system = False       # True if this a system theme
+        self._system_exists = False   # True if there exists a system
                                      #  theme with the same basename
-        self.name = ""
+        self._name = ""
 
         # create attributes
         for name, _type, default in self.attributes:
@@ -79,7 +84,12 @@ class Theme:
     @property
     def basename(self):
         """ Returns the file base name of the theme. """
-        return os.path.splitext(os.path.basename(self.filename))[0]
+        return os.path.splitext(os.path.basename(self._filename))[0]
+
+    @property
+    def filename(self):
+        """ Returns the filename of the theme. """
+        return self._filename
 
     def __eq__(self, other):
         if not other:
@@ -90,7 +100,7 @@ class Theme:
         return True
 
     def __str__(self):
-        return "name=%s, colors=%s, font=%s, radius=%d" % (self.name,
+        return "name=%s, colors=%s, font=%s, radius=%d" % (self._name,
                                                 self.color_scheme_basename,
                                                 self.key_label_font,
                                                 self.roundrect_radius)
@@ -100,7 +110,7 @@ class Theme:
         filename = self.get_color_scheme_filename()
         if not filename:
             _logger.error(_format("Color scheme for theme '{filename}' not found", \
-                                  filename=self.filename))
+                                  filename=self._filename))
             return False
 
         config.theme_settings.set_color_scheme_filename(filename, save)
@@ -236,8 +246,13 @@ class Theme:
         """
         Returns the full path names of all themes found in the given path.
         """
-        files = os.listdir(path)
         themes = []
+
+        try:
+            files = os.listdir(path)
+        except OSError:
+            files = []
+
         for filename in files:
             if filename.endswith(Theme.extension()):
                 themes.append(os.path.join(path, filename))
@@ -249,7 +264,7 @@ class Theme:
 
         result = None
 
-        _file = open(filename, encoding = "UTF-8")
+        _file = open_utf8(filename)
         try:
             domdoc = minidom.parse(_file).documentElement
             try:
@@ -305,7 +320,7 @@ class Theme:
 
                             setattr(theme, name, value)
 
-                theme.filename = filename
+                theme._filename = filename
                 theme.is_system = is_system
                 theme.system_exists = is_system
                 result = theme
@@ -327,8 +342,8 @@ class Theme:
 
     def save_as(self, basename, name):
         """ Save this theme under a new name. """
-        self.filename = self.build_user_filename(basename)
-        self.name = name
+        self._filename = self.build_user_filename(basename)
+        self._name = name
         self.save()
 
     def save(self):
@@ -337,7 +352,7 @@ class Theme:
         domdoc = minidom.Document()
         try:
             theme_element = domdoc.createElement("theme")
-            theme_element.setAttribute("name", self.name)
+            theme_element.setAttribute("name", self._name)
             theme_element.setAttribute("format", str(self.THEME_FORMAT))
             domdoc.appendChild(theme_element)
 
@@ -378,7 +393,9 @@ class Theme:
 
             pretty_xml = toprettyxml(domdoc)
 
-            with open(self.filename, "w", encoding = "UTF-8") as _file:
+            XDGDirs.assure_user_dir_exists(self.user_path())
+
+            with open_utf8(self._filename, "w") as _file:
                 if sys.version_info.major >= 3:
                     _file.write(pretty_xml)
                 else:
@@ -387,7 +404,7 @@ class Theme:
         except Exception as xxx_todo_changeme2:
             (ex) = xxx_todo_changeme2
             raise Exceptions.ThemeFileError(_("Error saving ")
-                + self.filename, chained_exception = ex)
+                + self._filename, chained_exception = ex)
         finally:
             domdoc.unlink()
 
@@ -406,31 +423,36 @@ class ColorScheme(object):
     # onboard 0.97, tree format, rule-based color matching
     COLOR_SCHEME_FORMAT_TREE   = Version(2, 0)
 
-    COLOR_SCHEME_FORMAT = COLOR_SCHEME_FORMAT_TREE
+    # onboard 0.99, added window colors
+    COLOR_SCHEME_WINDOW_COLORS   = Version(2, 1)
 
-    name = ""
-    filename = ""
-    is_system = False
-    root = None       # tree root
+    COLOR_SCHEME_FORMAT = COLOR_SCHEME_WINDOW_COLORS
 
     def __init__(self):
-        pass
+        self._filename = ""
+        self._is_system = False
+        self._root = None       # tree root
 
     @property
     def basename(self):
         """ Returns the file base name of the color scheme. """
-        return os.path.splitext(os.path.basename(self.filename))[0]
+        return os.path.splitext(os.path.basename(self._filename))[0]
 
-    def is_key_in_schema(self, key):
+    @property
+    def filename(self):
+        """ Returns the filename of the color scheme. """
+        return self._filename
+
+    def is_key_in_scheme(self, key):
         for id in [key.theme_id, key.id]:
-            if self.root.find_key_id(id):
+            if self._root.find_key_id(id):
                 return True
         return False
 
     def get_key_rgba(self, key, element, state = None):
         """
         Get the color for the given key element and optionally key state.
-        If state is None, the key state is taken from the key itself.
+        If <state> is None the key state is retrieved from <key>.
         """
 
         if state is None:
@@ -448,16 +470,34 @@ class ColorScheme(object):
         root_opacity = None
         key_group = None
 
-        # first try to find the theme_id then fall back to the generic id
-        for id in [key.theme_id, key.id]:
-            key_group = self.root.find_key_id(id)
+        # First try to find the theme_id then fall back to the generic id
+        ids = [key.theme_id, key.id]
+
+        # Let numbered keys fall back to their base id, e.g. instead
+        # of prediction0, prediction1,... have only "prediction" in
+        # the color scheme.
+        if key.id == "correctionsbg":
+            ids.append("wordlist")
+        elif key.id == "predictionsbg":
+            ids.append("wordlist")
+        elif key.is_prediction_key():
+            ids.append("prediction")
+        elif key.is_correction_key():
+            ids.append("correction")
+        elif key.is_layer_button():
+            ids.append(key.build_theme_id("layer"))
+            ids.append("layer")
+
+        # look for a matching key_group and color in the color scheme
+        for id in ids:
+            key_group = self._root.find_key_id(id)
             if key_group:
                 rgb, opacity = key_group.find_element_color(element, state)
                 break
 
         # Get root colors as fallback for the case when key id
         # wasn't mentioned anywhere in the color scheme.
-        root_key_group = self.root.get_default_key_group()
+        root_key_group = self._root.get_default_key_group()
         if root_key_group:
             root_rgb, root_opacity = \
                     root_key_group.find_element_color(element, state)
@@ -465,7 +505,8 @@ class ColorScheme(object):
         # Special case for layer buttons:
         # don't take fill color from the root group,
         # we want the layer fill color instead (via get_key_default_rgba()).
-        if element == "fill" and key.is_layer_button():
+        if element == "fill" and key.is_layer_button() or \
+           element == "label" and key.is_correction_key():
             # Don't pick layer fill opacity when there is
             # an rgb color defined in the color scheme.
             if not rgb is None and \
@@ -474,7 +515,7 @@ class ColorScheme(object):
                 if opacity is None:
                     opacity = 1.0
         elif key_group is None:
-            # All other colors fall back to the root groups colors
+            # All other colors fall back to the root group's colors
             rgb = root_rgb
             opacity = root_opacity
 
@@ -489,15 +530,17 @@ class ColorScheme(object):
 
     def get_key_default_rgba(self, key, element, state):
         colors = {
-                    "fill":                   [0.9,  0.85, 0.7, 1.0],
-                    "prelight":               [0.0,  0.0,  0.0, 1.0],
-                    "pressed":                [0.6,  0.6,  0.6, 1.0],
-                    "active":                 [0.5,  0.5,  0.5, 1.0],
-                    "locked":                 [1.0,  0.0,  0.0, 1.0],
-                    "scanned":                [0.45, 0.45, 0.7, 1.0],
-                    "stroke":                 [0.0,  0.0,  0.0, 1.0],
-                    "label":                  [0.0,  0.0,  0.0, 1.0],
-                    "dwell-progress":         [0.82, 0.19, 0.25, 1.0],
+                    "fill":                     [0.9,  0.85, 0.7, 1.0],
+                    "prelight":                 [0.0,  0.0,  0.0, 1.0],
+                    "pressed":                  [0.6,  0.6,  0.6, 1.0],
+                    "active":                   [0.5,  0.5,  0.5, 1.0],
+                    "locked":                   [1.0,  0.0,  0.0, 1.0],
+                    "scanned":                  [0.45, 0.45, 0.7, 1.0],
+                    "stroke":                   [0.0,  0.0,  0.0, 1.0],
+                    "label":                    [0.0,  0.0,  0.0, 1.0],
+                    "secondary-label":          [0.5,  0.5,  0.5, 1.0],
+                    "dwell-progress":           [0.82, 0.19, 0.25, 1.0],
+                    "correction-label":         [1.0,  0.5,  0.5, 1.0],
                     }
 
         rgba = [0.0, 0.0, 0.0, 1.0]
@@ -530,7 +573,7 @@ class ColorScheme(object):
 
             elif state.get("scanned"):
                 rgba = colors["scanned"]
-                # Make scanned active modifier keys stick out by mixing
+                # Make scanned active modifier keys stick out by blending
                 # scanned color with non-scanned color.
                 if state.get("active"): # includes locked
                     # inactive scanned color
@@ -544,7 +587,7 @@ class ColorScheme(object):
                     new_state["scanned"] = False
                     fill = self.get_key_rgba(key, element, new_state)
 
-                    # mix inactive scanned color with unscanned fill color
+                    # blend inactive scanned color with unscanned fill color
                     for i in range(4):
                         rgba[i] = (scanned[i] + fill[i]) / 2.0
 
@@ -561,22 +604,23 @@ class ColorScheme(object):
             rgba == colors["stroke"]
 
         elif element == "label":
-            rgba = colors["label"]
+
+            if key.is_correction_key():
+                rgba = colors["correction-label"]
+            else:
+                rgba = colors["label"]
 
             # dim label color for insensitive keys
             if state.get("insensitive"):
-                new_state = dict(list(state.items()))
-                new_state["insensitive"] = False
-                fill = self.get_key_rgba(key, "fill", new_state)
-                rgba = self.get_key_rgba(key, "label", new_state)
+                rgba = self._get_insensitive_color(key, state, element)
 
-                h, lf, s = colorsys.rgb_to_hls(*fill[:3])
-                h, ll, s = colorsys.rgb_to_hls(*rgba[:3])
+        elif element == "secondary-label":
 
-                # Leave only one third of the lightness difference
-                # between label and fill color.
-                amount = (ll - lf) * 2.0 / 3.0
-                rgba = brighten(-amount, *rgba)
+            rgba = colors["secondary-label"]
+
+            # dim label color for insensitive keys
+            if state.get("insensitive"):
+                rgba = self._get_insensitive_color(key, state, element)
 
         elif element == "dwell-progress":
             rgba = colors["dwell-progress"]
@@ -586,6 +630,52 @@ class ColorScheme(object):
 
         return rgba
 
+    def _get_insensitive_color(self, key, state, element):
+        new_state = state.copy()
+        new_state["insensitive"] = False
+        fill = self.get_key_rgba(key, "fill", new_state)
+        rgba = self.get_key_rgba(key, element, new_state)
+
+        h, lf, s = colorsys.rgb_to_hls(*fill[:3])
+        h, ll, s = colorsys.rgb_to_hls(*rgba[:3])
+
+        # Leave only one third of the lightness difference
+        # between label and fill color.
+        amount = (ll - lf) * 2.0 / 3.0
+        return brighten(-amount, *rgba)
+
+    def get_window_rgba(self, window_type, element):
+        """
+        Returns window colors.
+        window_type may be "keyboard" or "key-popup".
+        element may be "border"
+        """
+        rgb = None
+        opacity = None
+        windows = self._root.get_windows()
+
+        window = None
+        for item in windows:
+            if item.type == window_type:
+                window = item
+                break
+
+        if window:
+            for item in window.items:
+                if item.is_color() and \
+                   item.element == element:
+                    rgb = item.rgb
+                    opacity = item.opacity
+                    break
+
+        if rgb is None:
+            rgb = [1.0, 1.0, 1.0]
+        if opacity is None:
+            opacity = 1.0
+        rgba = rgb + [opacity]
+
+        return rgba
+
     def get_layer_fill_rgba(self, layer_index):
         """
         Returns the background fill color of the layer with the given index.
@@ -593,7 +683,11 @@ class ColorScheme(object):
 
         rgb = None
         opacity = None
-        layers = self.root.get_layers()
+        layers = self._root.get_layers()
+
+        # If there is no layer definition for this index,
+        # repeat the last defined layer color.
+        layer_index = min(layer_index, len(layers) - 1)
 
         if layer_index >= 0 and layer_index < len(layers):
             for item in layers[layer_index].items:
@@ -617,7 +711,7 @@ class ColorScheme(object):
         """
         rgb = None
         opacity = None
-        icons = self.root.get_icons()
+        icons = self._root.get_icons()
         for icon in icons:
             for item in icon.items:
                 if item.is_color() and \
@@ -650,7 +744,7 @@ class ColorScheme(object):
     @staticmethod
     def user_path():
         """ Returns the path of the user directory for color schemes. """
-        return os.path.join(config.user_dir, "themes/")
+        return os.path.join(config.user_dir, "themes")
 
     @staticmethod
     def system_path():
@@ -699,8 +793,13 @@ class ColorScheme(object):
         """
         Returns the full path names of all color schemes found in the given path.
         """
-        files = os.listdir(path)
         color_schemes = []
+
+        try:
+            files = os.listdir(path)
+        except OSError:
+            files = []
+
         for filename in files:
             if filename.endswith(ColorScheme.extension()):
                 color_schemes.append(os.path.join(path, filename))
@@ -708,11 +807,11 @@ class ColorScheme(object):
 
     @staticmethod
     def load(filename, is_system=False):
-        """ Load a color scheme and return it as a new object. """
+        """ Load a color scheme and return it as a new instance. """
 
         color_scheme = None
 
-        f = open(filename, encoding="UTF-8")
+        f = open_utf8(filename)
         try:
             dom = minidom.parse(f).documentElement
             name = dom.attributes["name"].value
@@ -741,9 +840,9 @@ class ColorScheme(object):
 
                 color_scheme = ColorScheme()
                 color_scheme.name = name
-                color_scheme.filename = filename
+                color_scheme._filename = filename
                 color_scheme.is_system = is_system
-                color_scheme.root = root
+                color_scheme._root = root
                 #print(root.dumps())
         except xml.parsers.expat.ExpatError as ex:
             _logger.error(_format("Error loading color scheme '{filename}'. "
@@ -762,10 +861,12 @@ class ColorScheme(object):
         items = []
         for child in dom_node.childNodes:
             if child.nodeType == minidom.Node.ELEMENT_NODE:
-                if child.tagName == "icon":
-                    item = ColorScheme._parse_icon(child)
+                if child.tagName == "window":
+                    item = ColorScheme._parse_window(child)
                 elif child.tagName == "layer":
                     item = ColorScheme._parse_layer(child)
+                elif child.tagName == "icon":
+                    item = ColorScheme._parse_icon(child)
                 elif child.tagName == "key_group":
                     item = ColorScheme._parse_key_group(child, used_keys)
                 elif child.tagName == "color":
@@ -787,8 +888,10 @@ class ColorScheme(object):
             item.id = node.attributes["id"].value
 
     @staticmethod
-    def _parse_icon(node):
-        item = Icon()
+    def _parse_window(node):
+        item = Window()
+        if node.hasAttribute("type"):
+            item.type = node.attributes["type"].value
         ColorScheme._parse_dom_node_item(node, item)
         return item
 
@@ -798,6 +901,14 @@ class ColorScheme(object):
         ColorScheme._parse_dom_node_item(node, item)
         return item
 
+    @staticmethod
+    def _parse_icon(node):
+        item = Icon()
+        ColorScheme._parse_dom_node_item(node, item)
+        return item
+
+    _key_ids_pattern = re.compile('[\w-]+(?:[.][\w-]+)?', re.UNICODE)
+
     @staticmethod
     def _parse_key_group(node, used_keys):
         item = KeyGroup()
@@ -806,7 +917,7 @@ class ColorScheme(object):
         # read key ids
         text = "".join([n.data for n in node.childNodes \
                         if n.nodeType == n.TEXT_NODE])
-        ids = [x for x in re.findall('\w+(?:[.][\w-]+)?', text) if x]
+        ids = [id for id in ColorScheme._key_ids_pattern.findall(text) if id]
 
         # check for duplicate key definitions
         for key_id in ids:
@@ -1002,7 +1113,6 @@ class ColorScheme(object):
             else:
                 key_groups.append(key_group)
 
-
         if root_key_group:
             root_key_group.append_items(key_groups)
             items.append(root_key_group)
@@ -1027,10 +1137,12 @@ class ColorSchemeItem(TreeItem):
         _level -= 1
         return s
 
-    def is_icon(self):
+    def is_window(self):
         return False
     def is_layer(self):
         return False
+    def is_icon(self):
+        return False
     def is_key_group(self):
         return False
     def is_color(self):
@@ -1053,6 +1165,17 @@ class ColorSchemeItem(TreeItem):
 class Root(ColorSchemeItem):
     """ Container for a layers colors """
 
+    def get_windows(self):
+        """
+        Get list of window in order of appearance
+        in the color scheme file.
+        """
+        windows = []
+        for item in self.items:
+            if item.is_window():
+                windows.append(item)
+        return windows
+
     def get_layers(self):
         """
         Get list of layer items in order of appearance
@@ -1083,20 +1206,29 @@ class Root(ColorSchemeItem):
         return None
 
 
-class Icon(ColorSchemeItem):
-    """ Container for a Icon's' colors """
+class Window(ColorSchemeItem):
+    """ Container for a window's colors """
+    
+    type = ""   # keyboard, key-popup
 
-    def is_icon(self):
+    def is_window(self):
         return True
 
 
 class Layer(ColorSchemeItem):
-    """ Container for a layers colors """
+    """ Container for a layer's colors """
 
     def is_layer(self):
         return True
 
 
+class Icon(ColorSchemeItem):
+    """ Container for a Icon's' colors """
+
+    def is_icon(self):
+        return True
+
+
 class Color(ColorSchemeItem):
     """ A single color, rgb + opacity """
     element = None
@@ -1159,7 +1291,7 @@ class KeyColor(Color):
                not attr in self.state:
                 default = False   # consider unspecified states to be False
 
-            if element == "label" and \
+            if (element == "label" or element == "secondary-label") and \
                attr in ["insensitive"] and \
                not attr in self.state:
                 default = False   # consider unspecified states to be False
