@@ -7,17 +7,19 @@ from __future__ import division, print_function, unicode_literals
 
 import os
 import sys
+import locale
 from shutil import copytree
 from optparse import OptionParser
 
-from gi.repository import GLib, Gtk, Gio
+from gi.repository import Gtk, Gio
 
 from Onboard.utils          import show_confirmation_dialog, Version, \
-                                   unicode_str, XDGDirs, chmodtree
-from Onboard.definitions    import InputEventSourceEnum, TouchInputEnum
+                                   unicode_str, XDGDirs, chmodtree, \
+                                   Process
+from Onboard.definitions    import *
 from Onboard.WindowUtils    import Handle, DockingEdge
 from Onboard.ConfigUtils    import ConfigObject
-from Onboard.ClickSimulator import CSMousetweaks
+from Onboard.ClickSimulator import CSMousetweaks0, CSMousetweaks1
 from Onboard.Exceptions     import SchemaError
 
 ### Logging ###
@@ -96,6 +98,11 @@ class Config(ConfigObject):
     Singleton Class to encapsulate the gsettings stuff and check values.
     """
 
+    # launched by ...
+    (LAUNCHER_NONE,
+     LAUNCHER_GSS,
+     LAUNCHER_UNITY_GREETER) = range(3)
+
     # extension of layout files
     LAYOUT_FILE_EXTENSION = ".onboard"
 
@@ -116,13 +123,16 @@ class Config(ConfigObject):
     SUPERKEY_SIZE_GROUP = "super"
 
     # width of frame around onboard when window decoration is disabled
-    UNDECORATED_FRAME_WIDTH = 5.0
+    UNDECORATED_FRAME_WIDTH = 6.0
+
+    # width of frame around popup windows
+    POPUP_FRAME_WIDTH = 5.0
 
     # radius of the rounded window corners
     CORNER_RADIUS = 10
 
     # y displacement of the key face of dish keys
-    DISH_KEY_Y_OFFSET = 1.0
+    DISH_KEY_Y_OFFSET = 0.8
 
     # raised border size of dish keys
     DISH_KEY_BORDER = (2.5, 2.5)
@@ -131,11 +141,14 @@ class Config(ConfigObject):
     UNPRESS_DELAY = 0.15
 
     # Margin to leave around wordlist labels; smaller margins leave
-    # room for more prediction choices
+    # more room for prediction choices
     WORDLIST_LABEL_MARGIN = (2, 2)
 
     # Gap between wordlist buttons
-    WORDLIST_BUTTON_SPACING = (1, 1)
+    WORDLIST_BUTTON_SPACING = (0.5, 0.5)
+
+    # Gap between wordlist predictions and correctios
+    WORDLIST_ENTRY_SPACING = (1.0, 1.0)
 
     # index of currently active pane, not stored in gsettings
     active_layer_index = 0
@@ -234,6 +247,13 @@ class Config(ConfigObject):
         self.gnome_a11y = self.add_optional_child(ConfigGDA)
         self.gnome_a11y.init_from_gsettings()
 
+        # detect who launched us
+        self.launched_by = self.LAUNCHER_NONE
+        if self.xid_mode:
+            if Process.was_launched_by("gnome-screensaver"):
+                self.launched_by = self.LAUNCHER_GSS
+            elif "UNITY_GREETER_DBUS_NAME" in os.environ:
+                self.launched_by = self.LAUNCHER_UNITY_GREETER
 
     def init(self):
         """
@@ -283,11 +303,42 @@ class Config(ConfigObject):
                     _logger.warning("migrating gsettings paths failed: " + \
                                     unicode_str(ex))
 
+        # Migrate old user language model to language specific
+        # model for Onboard 1.0.
+        from Onboard.WPEngine import ModelCache
+        old_fn = ModelCache.get_filename("lm:user:user")
+        if os.path.exists(old_fn):
+            lang_id = locale.getdefaultlocale()[0]
+            new_fn = ModelCache.get_filename("lm:user:" + lang_id)
+            old_bak = ModelCache.get_backup_filename(old_fn)
+            new_bak = ModelCache.get_backup_filename(new_fn)
+
+            _logger.info("Starting migration, user.lm has been deprecated.")
+            for old, new in [[old_fn, new_fn], [old_bak, new_bak]]:
+                if os.path.exists(old):
+                    if os.path.exists(new):
+                        _logger.info("Migration target already exists, "
+                                     "skipping renaming "
+                                     "'{}' to '{}'." \
+                                     .format(old, new))
+                        break # skip backup
+
+                    _logger.info("Migrating user language model "
+                                    "'{}' to '{}'." \
+                                    .format(old, new))
+                    try:
+                        os.rename(old, new)
+                    except OSError as ex:
+                        _logger.error("Failed to migrate "
+                                        "user language model. " + \
+                                        unicode_str(ex))
+                        break
+
         # Load system defaults (if there are any, not required).
         # Used for distribution specific settings, aka branding.
         paths = XDGDirs.get_all_config_dirs(USER_DIR) + \
-                [os.path.join(self.install_dir, SYSTEM_DEFAULTS_FILENAME),
-                 os.path.join("/etc/onboard", SYSTEM_DEFAULTS_FILENAME)]
+                [self.install_dir, "/etc/onboard"]
+        paths = [os.path.join(p, SYSTEM_DEFAULTS_FILENAME) for p in paths]
         self.load_system_defaults(paths)
 
         # initialize all property values
@@ -316,14 +367,10 @@ class Config(ConfigObject):
         # misc initializations
         self._last_snippets = dict(self.snippets)  # store a copy
 
-        # remember state of mousetweaks click-type window
+        # init state of mousetweaks' click-type window
         if self.mousetweaks:
-            self.mousetweaks.old_click_type_window_visible = \
-                          self.mousetweaks.click_type_window_visible
-
-            if self.mousetweaks.is_active() and \
-                self.universal_access.hide_click_type_window:
-                self.mousetweaks.click_type_window_visible = False
+            self.mousetweaks.init_click_type_window_visible(
+                self.universal_access.hide_click_type_window)
 
         # remember if we are running under GDM
         self.running_under_gdm = 'RUNNING_UNDER_GDM' in os.environ
@@ -347,15 +394,9 @@ class Config(ConfigObject):
 
     def final_cleanup(self):
         if self.mousetweaks:
-            if self.xid_mode:
-                self.mousetweaks.click_type_window_visible = \
-                        self.mousetweaks.old_click_type_window_visible
-            else:
-                if self.universal_access.enable_click_type_window_on_exit:
-                    self.mousetweaks.click_type_window_visible = True
-                else:
-                    self.mousetweaks.click_type_window_visible = \
-                        self.mousetweaks.old_click_type_window_visible
+            self.mousetweaks.restore_click_type_window_visible(
+                self.universal_access.enable_click_type_window_on_exit and \
+                not self.xid_mode)
 
     def _init_keys(self):
         """ Create key descriptions """
@@ -373,6 +414,10 @@ class Config(ConfigObject):
         self.add_key("system-theme-associations", {}, 'a{ss}')
         self.add_key("snippets", {}, "as")
         self.add_key("show-status-icon", True)
+        self.add_key("status-icon-provider", StatusIconProviderEnum.AppIndicator,
+                                             enum={"GtkStatusIcon" : 0,
+                                                   "AppIndicator" : 1,
+                                                  })
         self.add_key("start-minimized", False)
         self.add_key("xembed-onboard", False, prop="onboard_xembed_enabled")
         self.add_key("show-tooltips", True)
@@ -404,12 +449,18 @@ class Config(ConfigObject):
                           self.scanner,
                           self.typing_assistance]
 
-        try:
-            self.mousetweaks = CSMousetweaks()
-            self.children.append(self.mousetweaks)
-        except (SchemaError, ImportError) as e:
-            _logger.warning(unicode_str(e))
-            self.mousetweaks = None
+        for _class in [CSMousetweaks1, CSMousetweaks0]:
+            _class.MOUSETWEAKS_SCHEMA_ID
+            try:
+                self.mousetweaks = _class()
+                self.children.append(self.mousetweaks)
+                break
+            except (SchemaError, ImportError) as e:
+                _logger.info(unicode_str(e))
+                self.mousetweaks = None
+        if self.mousetweaks is None:
+            _logger.warning("mousetweaks GSettings schema not found, "
+                            "mousetweaks integration disabled.")
 
     def init_from_gsettings(self):
         """
@@ -501,7 +552,9 @@ class Config(ConfigObject):
     def _convert_sysdef_key(self, gskey, sysdef, value):
         # key exclusive to system defaults?
         if sysdef in ["superkey-label", \
-                      "superkey-label-independent-size"]:
+                      "superkey-label-independent-size",
+                      "xembed-aspect-change-range",
+                      "xembed-unity-greeter-offset-x"]:
             return value
         else:
             return super(self.__class__, self). \
@@ -685,32 +738,14 @@ class Config(ConfigObject):
     def get_system_model_dir(self):
         return os.path.join(self.install_dir, "models")
 
-    def allow_system_click_type_window(self, allow):
-        """ called from hover click button """
-        if not self.mousetweaks:
-            return
-
-        # This assumes that mousetweaks.click_type_window_visible never
-        # changes between activation and deactivation of mousetweaks.
-        if allow:
-            self.mousetweaks.click_type_window_visible = \
-                self.mousetweaks.old_click_type_window_visible
-        else:
-            # hide the mousetweaks window when onboard's settings say so
-            if self.universal_access.hide_click_type_window:
-
-                self.mousetweaks.old_click_type_window_visible = \
-                            self.mousetweaks.click_type_window_visible
-
-                self.mousetweaks.click_type_window_visible = False
-
     def enable_hover_click(self, enable):
+        hide = self.universal_access.hide_click_type_window
         if enable:
-            self.allow_system_click_type_window(False)
+            self.mousetweaks.allow_system_click_type_window(False, hide)
             self.mousetweaks.set_active(True)
         else:
             self.mousetweaks.set_active(False)
-            self.allow_system_click_type_window(True)
+            self.mousetweaks.allow_system_click_type_window(True, hide)
 
     def is_hover_click_active(self):
         return bool(self.mousetweaks) and self.mousetweaks.is_active()
@@ -808,8 +843,12 @@ class Config(ConfigObject):
     def is_inactive_transparency_enabled(self):
         return self.window.enable_inactive_transparency and \
                not self.scanner.enabled
+
     def is_keep_aspect_ratio_enabled(self):
-        return self.window.keep_aspect_ratio or self.options.keep_aspect_ratio
+        return self.window.keep_aspect_ratio or \
+               self.options.keep_aspect_ratio or \
+               (self.xid_mode and self.launched_by != self.LAUNCHER_NONE and \
+                not self.system_defaults.get("xembed_aspect_change_range") is None)
 
     def is_mousetweaks_active(self):
         return self.mousetweaks and self.mousetweaks.is_active()
@@ -936,6 +975,15 @@ class Config(ConfigObject):
     modeless_gksu = property(lambda self: False)
 
 
+    def get_desktop_background_filename(self):
+        try:
+            s = Gio.Settings(schema="org.gnome.desktop.background")
+            fn = s.get_string("picture-uri")
+        except:
+            fn = ""
+        fn = fn.replace("file://", "")
+        return fn
+
     def _get_install_dir(self):
         result = None
 
@@ -977,6 +1025,7 @@ class ConfigKeyboard(ConfigObject):
 
         self.add_key("show-click-buttons", False)
         self.add_key("sticky-key-release-delay", 0.0)
+        self.add_key("sticky-key-release-on-hide-delay", 5.0)
         self.add_key("sticky-key-behavior", {"all" : "cycle"}, 'a{ss}')
         self.add_key("long-press-delay", 0.5)
         self.add_key("default-key-action", self.DEFAULT_KEY_ACTION,
@@ -1198,15 +1247,9 @@ class ConfigUniversalAccess(ConfigObject):
     def _post_notify_hide_click_type_window(self):
         """ called when changed in gsettings (preferences window) """
         mousetweaks = self.parent.mousetweaks
-
-        if not mousetweaks:
-            return
-        if mousetweaks.is_active():
-            if self.hide_click_type_window:
-                mousetweaks.click_type_window_visible = False
-            else:
-                mousetweaks.click_type_window_visible = \
-                            mousetweaks.old_click_type_window_visible
+        if mousetweaks:
+            mousetweaks.on_hide_click_type_window_changed(
+                self.hide_click_type_window)
 
 
 class ConfigTheme(ConfigObject):
@@ -1410,11 +1453,16 @@ class ConfigWordSuggestions(ConfigObject):
         self.add_key("enabled", False)
         self.add_key("auto-learn", True)
         self.add_key("punctuation-assistance", True)
-        self.add_key("stealth-mode", False)
         self.add_key("accent-insensitive", True)
         self.add_key("max-word-choices", 5)
         self.add_key("spelling-suggestions-enabled", True)
+        self.add_key("wordlist-buttons", ["language", "hide"])
+        self.add_key("pause-learning-locked", False)
 
+        self._pause_learning = 0  # 0=off, 1=latched, 2=locked; not in gsettings
+
+        # deprecated
+        self.add_key("stealth-mode", False)
         self.add_key("show-context-line", False)
 
     def word_prediction_notify_add(self, callback):
@@ -1425,6 +1473,68 @@ class ConfigWordSuggestions(ConfigObject):
     def can_auto_learn(self):
         return self.enabled and \
                self.auto_learn and \
+               not self.get_pause_learning() and \
                not self.stealth_mode
 
+    def get_pause_learning(self):
+        if self.pause_learning_locked:
+            return 2
+        else:
+            return self._pause_learning
+
+    def set_pause_learning(self, value):
+        self._pause_learning = value
+        self.pause_learning_locked = value == 2
+
+    def _post_notify_pause_learning_locked(self):
+        self._pause_learning = 2 if self.pause_learning_locked else 0
+
+    KEY_ID_LANGUAGE = "language"
+    KEY_ID_PAUSE_LEARNING = "pause-learning"
+
+    def get_shown_wordlist_button_ids(self):
+        result = []
+        for button_id in self.wordlist_buttons:
+            if button_id != self.KEY_ID_PAUSE_LEARNING or \
+                self.can_show_pause_learning_button():
+                result.append(button_id)
+        return result
+
+    def can_show_language_button(self):
+        return self.KEY_ID_LANGUAGE in self.wordlist_buttons
+
+    def show_language_button(self, show):
+        key_id = self.KEY_ID_LANGUAGE
+        buttons = self.wordlist_buttons[:]
+        if show:
+            if not key_id in buttons:
+                try:
+                    index = buttons.index("hide")
+                    buttons.insert(index, key_id)
+                except ValueError:
+                    buttons.append(key_id)
+
+                self.wordlist_buttons = buttons
+        else:
+            if key_id in buttons:
+                buttons.remove(key_id)
+                self.wordlist_buttons = buttons
+
+    def can_show_pause_learning_button(self):
+        return self.auto_learn and \
+               self.KEY_ID_PAUSE_LEARNING in self.wordlist_buttons
+
+    def show_pause_learning_button(self, show):
+        key_id = self.KEY_ID_PAUSE_LEARNING
+        buttons = self.wordlist_buttons[:]
+        if show:
+            if not key_id in buttons:
+                buttons.insert(0, key_id)
+
+                self.wordlist_buttons = buttons
+        else:
+            if key_id in buttons:
+                buttons.remove(key_id)
+                self.wordlist_buttons = buttons
+
 
