@@ -22,7 +22,7 @@ from __future__ import division, print_function, unicode_literals
 
 from os.path import join
 from traceback import print_exc
-
+  
 from gi.repository import GObject, Gdk, Gtk
 
 import cairo
@@ -32,6 +32,7 @@ from Onboard.utils       import CallOnce, Rect, round_corners, roundrect_arc, \
 from Onboard.WindowUtils import WindowManipulator, WindowRectTracker, \
                                 Orientation, set_unity_property, \
                                 DwellProgress
+from Onboard.TouchInput  import InputSequence, POINTER_SEQUENCE
 from Onboard.KeyGtk      import RectKey
 
 ### Logging ###
@@ -64,7 +65,7 @@ class IconPalette(Gtk.Window, WindowRectTracker, WindowManipulator):
     """ Minimum size of the IconPalette """
     MINIMUM_SIZE = 20
 
-    _keyboard = None
+    _layout_view = None
 
     def __init__(self):
 
@@ -120,7 +121,7 @@ class IconPalette(Gtk.Window, WindowRectTracker, WindowManipulator):
         self.restore_window_rect()
 
         # Realize the window. Test changes to this in all supported
-        # environments. It's all to easy to make the icp not show up reliably.
+        # environments. It's all too easy to have the icp not show up reliably.
         self.update_window_options()
         self.hide()
 
@@ -137,13 +138,13 @@ class IconPalette(Gtk.Window, WindowRectTracker, WindowManipulator):
     def cleanup(self):
         WindowRectTracker.cleanup(self)
 
-    def set_keyboard(self, keyboard):
-        self._keyboard = keyboard
+    def set_layout_view(self, view):
+        self._layout_view = view
         self.queue_draw()
 
     def get_color_scheme(self):
-        if self._keyboard:
-            return self._keyboard.color_scheme
+        if self._layout_view:
+            return self._layout_view.get_color_scheme()
         return None
 
     def _on_configure_event(self, widget, event):
@@ -161,7 +162,7 @@ class IconPalette(Gtk.Window, WindowRectTracker, WindowManipulator):
     def _on_realize_event(self, user_data):
         """ Gdk window created """
         set_unity_property(self)
-        if config.window.force_to_top:
+        if config.is_force_to_top():
             self.get_window().set_override_redirect(True)
         self.restore_window_rect(True)
 
@@ -170,7 +171,7 @@ class IconPalette(Gtk.Window, WindowRectTracker, WindowManipulator):
         self.set_type_hint(self._get_window_type_hint())
 
     def _get_window_type_hint(self):
-        if config.window.force_to_top:
+        if config.is_force_to_top():
             return Gdk.WindowTypeHint.NORMAL
         else:
             return Gdk.WindowTypeHint.UTILITY
@@ -179,7 +180,7 @@ class IconPalette(Gtk.Window, WindowRectTracker, WindowManipulator):
         if not config.xid_mode:   # not when embedding
 
             # (re-)create the gdk window?
-            force_to_top = config.window.force_to_top
+            force_to_top = config.is_force_to_top()
 
             if self._force_to_top != force_to_top:
                 self._force_to_top = force_to_top
@@ -216,7 +217,9 @@ class IconPalette(Gtk.Window, WindowRectTracker, WindowManipulator):
         """
         if event.button == 1 and event.window == self.get_window():
             self.enable_drag_protection(True)
-            self.handle_press(event, move_on_background = True)
+            sequence = InputSequence()
+            sequence.init_from_button_event(event)
+            self.handle_press(sequence, move_on_background = True)
             if self.is_moving():
                 self.reset_drag_protection() # force threshold
         return False
@@ -225,7 +228,9 @@ class IconPalette(Gtk.Window, WindowRectTracker, WindowManipulator):
         """
         Move the window if the pointer has moved more than the DND threshold.
         """
-        self.handle_motion(event, fallback = True)
+        sequence = InputSequence()
+        sequence.init_from_motion_event(event)
+        self.handle_motion(sequence, fallback = True)
         self.set_drag_cursor_at((event.x, event.y))
 
         # start dwelling if nothing else is going on
@@ -342,7 +347,7 @@ class IconPalette(Gtk.Window, WindowRectTracker, WindowManipulator):
 
         themed = False
         if color_scheme:
-            if any(color_scheme.is_key_in_schema(key) for key in keys):
+            if any(color_scheme.is_key_in_scheme(key) for key in keys):
                 themed = True
 
         # four rounded rectangles
@@ -434,9 +439,9 @@ class IconPalette(Gtk.Window, WindowRectTracker, WindowManipulator):
         else:
             co = config.icp.portrait
 
-        co.settings.delay()
+        co.delay()
         co.x, co.y, co.width, co.height = rect
-        co.settings.apply()
+        co.apply()
 
     def _is_dwelling(self):
         return bool(self._dwell_begin_timer) and \
