@@ -113,9 +113,18 @@ class KbdWindowBase:
         self.check_alpha_support()
         self.queue_draw()
 
-    def detect_window_manager(self):
-        """ Detect the WM and select WM specific behavior. """
+    def emit_quit_onboard(self):
+        self.emit("quit-onboard")
 
+    def detect_window_manager(self):
+        """
+        Detect the WM and select WM specific behavior.
+
+        Doctests:
+        >>> osk_util = osk.Util()
+        >>> len(osk_util.get_current_wm_name().lower()) > 0
+        True
+        """
         wm = config.quirks
         if not wm:
             # Returns None on X error BadWindow (LP: 1016980)
@@ -140,7 +149,9 @@ class KbdWindowBase:
     def check_alpha_support(self):
         screen = self.get_screen()
         visual = screen.get_rgba_visual()
-        self.supports_alpha = visual and screen.is_composited()
+        self.supports_alpha = visual and \
+                              (screen.is_composited() or \
+                               config.running_under_gdm) # for lightdm
 
         self.keyboard_widget.supports_alpha = self.supports_alpha
 
@@ -222,17 +233,17 @@ class KbdWindowBase:
 
             recreate = False
 
-            # Window decoration?
+            # Window decoration toggled?
             decorated = config.window.window_decoration
             if decorated != self.get_decorated():
                 recreate = True
 
-            # override redirect?
+            # Override redirect toggled?
             ord = self.is_override_redirect_mode()
             if ord != self._override_redirect:
                 recreate = True
 
-            # override redirect?
+            # Window type hint changed?
             type_hint = self._wm_quirks.get_window_type_hint(self)
             if type_hint != self._type_hint:
                 recreate = True
@@ -598,6 +609,7 @@ class KbdWindow(KbdWindowBase, WindowRectTracker, Gtk.Window):
             rect = self.get_current_rect()
             if not rect is None: # shouldn't happen, fix this
                 self.move_resize(*rect) # sync position
+                self.keyboard_widget.sync_transition_position(rect)
 
         KbdWindowBase.on_visibility_changed(self, visible)
 
@@ -1081,15 +1093,12 @@ class KbdWindow(KbdWindowBase, WindowRectTracker, Gtk.Window):
             return self.keyboard_widget.limit_size(rect)
         return rect
 
-    def _emit_quit_onboard(self, event, data=None):
-        self.emit("quit-onboard")
-
     def _on_delete_event(self, event, data=None):
         if config.lockdown.disable_quit:
             if self.keyboard_widget:
                 return True
         else:
-            self._emit_quit_onboard(event)
+            self.emit_quit_onboard()
 
     def on_docking_notify(self):
         self.update_docking()
@@ -1347,7 +1356,7 @@ class WMQuirksCompiz(WMQuirksDefault):
 
     @staticmethod
     def can_set_override_redirect(window):
-        # Cant't type into unity dash with keys below dash without
+        # Can't type into unity dash with keys below dash without
         # override redirect. -> turn it on even when docking. Apparently
         # Compiz can handle struts with OR windows.
         return True
