@@ -2,16 +2,17 @@
 
 from __future__ import division, print_function, unicode_literals
 
-import time
-from math import pi, sin, cos
+from math import pi, sin, cos, sqrt
 
 import cairo
 from gi.repository import GLib, Gdk, Pango, PangoCairo, GdkPixbuf
 
 from Onboard.KeyCommon   import *
 from Onboard.WindowUtils import DwellProgress
-from Onboard.utils       import brighten, roundrect_curve, gradient_line, \
-                                drop_shadow
+from Onboard.utils       import brighten, unicode_str, \
+                                gradient_line, drop_shadow, \
+                                roundrect_curve, rounded_path, \
+                                rounded_polygon_path_to_cairo_path
 
 ### Logging ###
 import logging
@@ -28,6 +29,7 @@ PangoUnscale = 1.0 / Pango.SCALE
 class Key(KeyCommon):
     _pango_layouts = None
     _label_extents = None  # resolution independent size {mod_mask: (w, h)}
+    _popup_indicator = ""  # font dependent popup indicator (ellipsis)
 
     _shadow_steps  = 0
     _shadow_alpha  = 0
@@ -120,17 +122,17 @@ class RectKey(Key, RectKeyCommon, DwellProgress):
             super(RectKey, self).set_border_rect(rect)
             self.invalidate_caches()
 
-    def draw_cached(self, context):
+    def draw_cached(self, cr):
         key = (self.label, self.font_size >> 8)
         surface = self._key_surfaces.get(key)
         if surface is None:
             if self.font_size:
-                surface = self._create_key_surface(context)
+                surface = self._create_key_surface(cr)
                 self._key_surfaces[key] = surface
 
         if surface:
-            context.set_source_surface(surface, 0, 0)
-            context.paint()
+            cr.set_source_surface(surface, 0, 0)
+            cr.paint()
 
     def _create_key_surface(self, base_context):
         rect = self.get_canvas_rect()
@@ -141,27 +143,26 @@ class RectKey(Key, RectKeyCommon, DwellProgress):
         surface = target.create_similar(cairo.CONTENT_COLOR_ALPHA,
                                         clip_rect.w, clip_rect.h)
 
-        context = cairo.Context(surface)
+        cr = cairo.Context(surface)
         surface.set_device_offset(-clip_rect.x, -clip_rect.y)
 
-        self.draw(context)
+        self.draw(cr)
         Gdk.flush()   # else artefacts in labels and images
                       # on Nexus 7, Raring
 
         return surface
 
-    def draw(self, context, lod = LOD.FULL):
-        self.draw_geometry(context, lod)
-        self.draw_image(context, lod)
-        self.draw_label(context, lod)
+    def draw(self, cr, lod = LOD.FULL):
+        self.draw_geometry(cr, lod)
+        self.draw_image(cr, lod)
+        self.draw_label(cr, lod)
 
-    def draw_geometry(self, context, lod):
+    def draw_geometry(self, cr, lod):
         if not self.show_face and not self.show_border:
             return
 
-        rect = self.get_canvas_rect()
         if lod == LOD.FULL and self.show_border:
-            scale = config.theme_settings.key_stroke_width / 100.0
+            scale = self.get_stroke_width()
             if scale:
                 root = self.get_layout_root()
                 t    = root.context.scale_log_to_canvas((1.0, 1.0))
@@ -177,33 +178,37 @@ class RectKey(Key, RectKeyCommon, DwellProgress):
 
         key_style = self.get_style()
         if key_style == "flat":
-            self.build_rect_path(context, rect)
-
-            if self.show_face:
-                context.set_source_rgba(*fill)
-                if line_width:
-                    context.fill_preserve()
-                else:
-                    context.fill()
-
-            if line_width:
-                context.set_source_rgba(*self.get_stroke_color())
-                context.set_line_width(line_width)
-                context.stroke()
+            self.draw_flat_key(cr, fill, line_width)
 
         elif key_style == "gradient":
-            self.draw_gradient_key(context, rect, fill, line_width, lod)
+            self.draw_gradient_key(cr, fill, line_width, lod)
 
         elif key_style == "dish":
-            self.draw_dish_key(context, rect, fill, line_width, lod)
+            self.draw_dish_key(cr, fill, line_width, lod)
 
-    def draw_gradient_key(self, context, rect, fill, line_width, lod):
+    def draw_flat_key(self, cr, fill, line_width):
+        self._build_canvas_path(cr)
+
+        if self.show_face:
+            cr.set_source_rgba(*fill)
+            if line_width:
+                cr.fill_preserve()
+            else:
+                cr.fill()
+
+        if line_width:
+            cr.set_source_rgba(*self.get_stroke_color())
+            cr.set_line_width(line_width)
+            cr.stroke()
+
+    def draw_gradient_key(self, cr, fill, line_width, lod):
         # simple gradients for fill and stroke
         fill_gradient   = config.theme_settings.key_fill_gradient / 100.0
-        stroke_gradient = config.theme_settings.key_stroke_gradient / 100.0
+        stroke_gradient = self.get_stroke_gradient()
         alpha = self.get_gradient_angle()
 
-        self.build_rect_path(context, rect)
+        rect = self.get_canvas_rect()
+        self._build_canvas_path(cr, rect)
         gline = gradient_line(rect, alpha)
 
         # fill
@@ -214,14 +219,14 @@ class RectKey(Key, RectKeyCommon, DwellProgress):
                 pat.add_color_stop_rgba(0, *rgba)
                 rgba = brighten(-fill_gradient*.5, *fill)
                 pat.add_color_stop_rgba(1, *rgba)
-                context.set_source (pat)
+                cr.set_source (pat)
             else: # take gradient from color scheme (not implemented)
-                context.set_source_rgba(*fill)
+                cr.set_source_rgba(*fill)
 
             if self.show_border:
-                context.fill_preserve()
+                cr.fill_preserve()
             else:
-                context.fill()
+                cr.fill()
 
         # stroke
         if self.show_border:
@@ -233,132 +238,92 @@ class RectKey(Key, RectKeyCommon, DwellProgress):
                     pat.add_color_stop_rgba(0, *rgba)
                     rgba = brighten(-stroke_gradient*.5, *stroke)
                     pat.add_color_stop_rgba(1, *rgba)
-                    context.set_source (pat)
+                    cr.set_source (pat)
                 else:
-                    context.set_source_rgba(*fill)
+                    cr.set_source_rgba(*fill)
             else:
-                context.set_source_rgba(*self.get_stroke_color())
+                cr.set_source_rgba(*self.get_stroke_color())
 
-            context.set_line_width(line_width)
-            context.stroke()
+            cr.set_line_width(line_width)
+        cr.stroke()
+
+    def draw_dish_key(self, cr, fill, line_width, lod):
+        canvas_rect = self.get_canvas_rect()
+        if self.geometry:
+            geometry = self.geometry
+        else:
+            geometry = KeyGeometry.from_rect(self.get_border_rect())
+        size_scale_x, size_scale_y = geometry.scale_log_to_size((1.0, 1.0))
 
-    def draw_dish_key(self, context, rect, fill, line_width, lod):
         # compensate for smaller size due to missing stroke
-        rect = rect.inflate(1.0)
-
-        # parameters for the base rectangle
-        w, h = rect.get_size()
-        w2, h2 = w * 0.5, h * 0.5
-        xc, yc = rect.get_center()
-        radius_pct = config.theme_settings.roundrect_radius
-        radius_pct = max(radius_pct, 2) # too much +-1 fudging for square corners
-        r, k = self.get_curved_rect_params(rect, radius_pct)
+        canvas_rect = canvas_rect.inflate(1.0)
 
+        # parameters for the base path
         base_rgba = brighten(-0.200, *fill)
-        stroke_gradient = config.theme_settings.key_stroke_gradient / 100.0
-        light_dir = self.get_light_direction()
+        stroke_gradient = self.get_stroke_gradient()
+        light_dir = self.get_light_direction() - pi * 0.5  # 0 = light from top
+        lightx = cos(light_dir)
+        lighty = sin(light_dir)
 
-        # parameters for the top rectangle, key face
-        scale  = config.theme_settings.key_stroke_width / 100.0
+        key_offset_x, key_offset_y, key_size_x, key_size_y = \
+                                            self.get_key_offset_size(geometry)
+        radius_pct = max(config.theme_settings.roundrect_radius, 2)
+        radius_pct = max(radius_pct, 2) # too much +-1 fudging for square corners
+        chamfer_size = self.get_chamfer_size()
+        chamfer_size = (self.context.scale_log_to_canvas_x(chamfer_size) +
+                        self.context.scale_log_to_canvas_y(chamfer_size)) * 0.5
+
+        # parameters for the top path, key face
+        stroke_width  = self.get_stroke_width()
+        key_offset_top_y = key_offset_y - \
+            config.DISH_KEY_Y_OFFSET * stroke_width
         border = config.DISH_KEY_BORDER
-        border = (border[0] * scale, border[1] * scale)
+        scale_top_x = 1.0 - (border[0] * stroke_width * size_scale_x * 2.0)
+        scale_top_y = 1.0 - (border[1] * stroke_width * size_scale_y * 2.0)
+        key_size_top_x = key_size_x * scale_top_x
+        key_size_top_y = key_size_y * scale_top_y
+        chamfer_size_top = chamfer_size * (scale_top_x + scale_top_y) * 0.5
 
-        border = self.context.scale_log_to_canvas(border)
-        offset_top = self.context.scale_log_to_canvas_y(config.DISH_KEY_Y_OFFSET)
-        rect_top = rect.deflate(*border).offset(0, -offset_top)
-        rect_top.w = max(rect_top.w, 0.0)
-        rect_top.h = max(rect_top.h, 0.0)
-        top_radius_scale = rect_top.h / float(rect.h)
-        r_top, k_top = self.get_curved_rect_params(rect_top,
-                                                radius_pct * top_radius_scale)
+        # realize all paths we're going to use
+        polygons, polygon_paths = \
+            self.get_canvas_polygons(geometry,
+                                   key_offset_x, key_offset_y,
+                                   key_size_x, key_size_y,
+                                   radius_pct, chamfer_size)
+        polygons_top, polygon_paths_top = \
+            self.get_canvas_polygons(geometry,
+                                   key_offset_x, key_offset_top_y,
+                                   key_size_top_x - size_scale_x,
+                                   key_size_top_y - size_scale_y,
+                                   radius_pct, chamfer_size_top)
+        polygons_top1, polygon_paths_top1 = \
+            self.get_canvas_polygons(geometry,
+                                   key_offset_x, key_offset_top_y,
+                                   key_size_top_x, key_size_top_y,
+                                   radius_pct, chamfer_size_top)
 
         # draw key border
         if self.show_border:
             if not lod:
-                self.build_rect_path(context, rect)
-                context.set_source_rgba(*base_rgba)
-                context.fill()
+                cr.set_source_rgba(*base_rgba)
+                for path in polygon_paths:
+                    rounded_polygon_path_to_cairo_path(cr, path)
+                    cr.fill()
             else:
+                for ipg, polygon in enumerate(polygons):
+                    polygon_top = polygons_top[ipg]
+                    path = polygon_paths[ipg]
+                    path_top = polygon_paths_top[ipg]
 
-                # lambert lighting
-                edge_colors = []
-                for edge in range(4):
-                    normal_dir = edge * pi / 2.0   # 0 = light from top
-                    I = cos(normal_dir - light_dir) * stroke_gradient * 0.8
-                    edge_colors.append(brighten(I, *base_rgba))
-
-                context.save()
-                context.translate(xc , yc)
-
-                # edge sections, edge 0 = top
-                for edge in range(4):
-                    if edge & 1:
-                        p = (h2, w2)
-                        p_top = [rect_top.h/2.0, rect_top.w/2.0]
-                    else:
-                        p = (w2, h2)
-                        p_top = [rect_top.w/2.0, rect_top.h/2.0]
-
-                    m = cairo.Matrix()
-                    m.rotate(edge * pi / 2.0)
-                    p0     = m.transform_point(-p[0] + r - 1, -p[1]) # -1 to fill gaps
-                    p1     = m.transform_point( p[0] - r + 1, -p[1])
-                    p0_top = m.transform_point( p_top[0] - r_top + 1, -p_top[1] + 1)
-                    p1_top = m.transform_point(-p_top[0] + r_top - 1, -p_top[1] + 1)
-                    p0_top = (p0_top[0], p0_top[1] - offset_top)
-                    p1_top = (p1_top[0], p1_top[1] - offset_top)
-
-                    context.set_source_rgba(*edge_colors[edge])
-                    context.move_to(p0[0], p0[1])
-                    context.line_to(p1[0], p1[1])
-                    context.line_to(*p0_top)
-                    context.line_to(*p1_top)
-                    context.close_path()
-                    context.fill()
-
-
-                # corner sections
-                for edge in range(4):
-                    if edge & 1:
-                        p = (h2, w2)
-                        p_top = [rect_top.h/2.0, rect_top.w/2.0]
-                    else:
-                        p = (w2, h2)
-                        p_top = [rect_top.w/2.0, rect_top.h/2.0]
-
-                    m = cairo.Matrix()
-                    m.rotate(edge * pi / 2.0)
-                    p1     = m.transform_point( p[0] - r, -p[1])
-                    p2     = m.transform_point( p[0],     -p[1] + r)
-                    pk0    = m.transform_point( p[0] - k, -p[1])
-                    pk1    = m.transform_point( p[0],     -p[1] + k)
-                    p0_top = m.transform_point( p_top[0] - r_top, -p_top[1])
-                    p2_top = m.transform_point( p_top[0],         -p_top[1] + r_top)
-                    p0_top = (p0_top[0], p0_top[1] - offset_top)
-                    p2_top = (p2_top[0], p2_top[1] - offset_top)
-
-                    # Fake Gouraud shading: draw a gradient between mid points
-                    # of the lines connecting the base with the top rectangle.
-                    gline = ((p1[0] + p0_top[0]) / 2.0, (p1[1] + p0_top[1]) / 2.0,
-                             (p2[0] + p2_top[0]) / 2.0, (p2[1] + p2_top[1]) / 2.0)
-                    pat = cairo.LinearGradient (*gline)
-                    pat.add_color_stop_rgba(0.0, *edge_colors[edge])
-                    pat.add_color_stop_rgba(1.0, *edge_colors[(edge + 1) % 4])
-                    context.set_source (pat)
-
-                    context.move_to(*p1)
-                    context.curve_to(pk0[0], pk0[1], pk1[0], pk1[1], p2[0], p2[1])
-                    context.line_to(*p2_top)
-                    context.line_to(*p0_top)
-                    context.close_path()
-                    context.fill()
-
-                context.restore()
+                    self._draw_dish_key_border(cr, path, path_top,
+                                               polygon, polygon_top,
+                                               base_rgba, stroke_gradient,
+                                               lightx, lighty)
 
         # Draw the key face, the smaller top rectangle.
         if self.show_face:
             if not lod:
-                context.set_source_rgba(*fill)
+                cr.set_source_rgba(*fill)
             else:
                 # Simulate the concave key dish with a gradient that has
                 # a sligthly brighter middle section.
@@ -369,16 +334,132 @@ class RectKey(Key, RectKeyCommon, DwellProgress):
                 fill_gradient   = config.theme_settings.key_fill_gradient / 100.0
                 dark_rgba = brighten(-fill_gradient*.5, *fill)
                 bright_rgba = brighten(+fill_gradient*.5, *fill)
-                gline = gradient_line(rect, angle)
+                gline = gradient_line(canvas_rect, angle)
 
                 pat = cairo.LinearGradient (*gline)
                 pat.add_color_stop_rgba(0.0, *dark_rgba)
                 pat.add_color_stop_rgba(0.5, *bright_rgba)
                 pat.add_color_stop_rgba(1.0, *dark_rgba)
-                context.set_source (pat)
+                cr.set_source (pat)
 
-            self.build_rect_path(context, rect_top, top_radius_scale)
-            context.fill()
+            for path in polygon_paths_top1:
+                rounded_polygon_path_to_cairo_path(cr, path)
+                cr.fill()
+
+    def _draw_dish_key_border(self, cr, path, path_top,
+                              polygon, polygon_top,
+                              base_rgba, stroke_gradient, lightx, lighty):
+        n = len(polygon)
+        m = len(path)
+
+        # Lambert lighting
+        edge_colors = []
+        for i in range(0, n, 2):
+            x0 = polygon[i]
+            y0 = polygon[i+1]
+            if i < n-2:
+                x1 = polygon[i+2]
+                y1 = polygon[i+3]
+            else:
+                x1 = polygon[0]
+                y1 = polygon[1]
+
+            nx = y1 - y0
+            ny = -(x1 - x0)
+            ln = sqrt(nx*nx + ny*ny)
+            I = (nx * lightx + ny * lighty) / ln \
+                * stroke_gradient * 0.8 \
+                if ln else 0.0
+            edge_colors.append(brighten(I, *base_rgba))
+
+        # draw border sections
+        edge = 0
+        for i in range(0, m-2, 2):
+            # get path points
+            i1 = i + 1
+            i2 = i + 2
+            if i2 >= m:
+                i2 -= m
+            i3 = i + 3
+            if i3 >= m:
+                i3 = 1
+
+            p0 = path[i]
+            p0x = p0[0]
+            p0y = p0[1]
+
+            p1 = path[i1]
+            p1x = p1[0]
+            p1y = p1[1]
+
+            p2 = path[i2]
+            p2x = p2[0]
+            p2y = p2[1]
+
+            p3 = path[i3]
+            p3x = p3[0]
+            p3y = p3[1]
+
+            p = path_top[i]
+            ptop0x = p[0]
+            ptop0y = p[1]
+
+            p = path_top[i1]
+            ptop1x = p[0]
+            ptop1y = p[1]
+
+            p = path_top[i2]
+            ptop2x = p[0]
+            ptop2y = p[1]
+
+            # get polygon points, only to
+            # fill in gaps at concave corners.
+            j0 = edge*2
+            j1 = j0 + 2
+            if j1 >= n:
+                j1 -= n
+            j2 = j0 + 4
+            if j2 >= n:
+                j2 -= n
+
+            ptopax = polygon_top[j0]
+            ptopay = polygon_top[j0 + 1]
+            ptopbx = polygon_top[j1]
+            ptopby = polygon_top[j1 + 1]
+            ptopcx = polygon_top[j2]
+            ptopcy = polygon_top[j2 + 1]
+            vax = ptopbx - ptopax
+            vay = ptopby - ptopay
+            nbx = ptopcy - ptopby
+            nby = -(ptopcx - ptopbx)
+            concave = vax*nbx + vay*nby < 0.0
+
+            # Fake Gouraud shading: draw a gradient between mid points
+            # of the lines connecting the base with the top path.
+            pat = cairo.LinearGradient((p1x + ptop1x) * 0.5,
+                                        (p1y + ptop1y) * 0.5,
+                                        (p2x + ptop2x) * 0.5,
+                                        (p2y + ptop2y) * 0.5)
+            edge1 = (edge + 1) % len(edge_colors)
+            pat.add_color_stop_rgba(0.0, *edge_colors[edge])
+            pat.add_color_stop_rgba(1.0, *edge_colors[edge1])
+            cr.set_source (pat)
+
+            # Draw corners and edges with enough overlap to avoid
+            # artefacts at touching line boundaries.
+            cr.move_to(p0x, p0y)
+            cr.line_to(p1x, p1y)
+            cr.curve_to(p2[2], p2[3], p2[4], p2[5], p2[0], p2[1])
+            cr.line_to(p3x, p3y)
+            cr.line_to(ptop2x, ptop2y)
+            if concave:
+                cr.line_to(ptopbx, ptopby)
+            cr.line_to(ptop1x, ptop1y)
+            cr.line_to(ptop0x, ptop0y)
+            cr.close_path()
+            cr.fill()
+
+            edge += 1
 
     def get_label_runs(self):
         runs = []
@@ -405,10 +486,11 @@ class RectKey(Key, RectKeyCommon, DwellProgress):
         # popup indicator
         if not self.popup_id is None and \
            not config.xid_mode:
-            label = "…"
-            label = "︙"
+
+            label = self._get_popup_indicator()
             font_size = self.font_size
             layout = self.get_pango_layout(label, font_size, 2)
+
             src_size = layout.get_size()
             src_size = (src_size[0] * PangoUnscale, src_size[1] * PangoUnscale)
             xalign, yalign = self.align_popup_indicator(src_size,
@@ -436,6 +518,28 @@ class RectKey(Key, RectKeyCommon, DwellProgress):
 
         return runs
 
+    def _get_popup_indicator(self):
+        """
+        Find the shortest ellipsis possible with the current font.
+        The font is assumed to never change during the livetime of the key.
+        """
+        result = self._popup_indicator
+        if not result:
+            labels = ("…", "...")  # label candidates
+
+            BASE_FONTDESCRIPTION_SIZE = 10000000
+            wmin = None
+            result = ""
+            for label in labels:
+                layout = self.get_pango_layout(label, BASE_FONTDESCRIPTION_SIZE, 2)
+                w = layout.get_size()[0]
+                if wmin is None or w < wmin:
+                    wmin = w
+                    result = label
+            self._popup_indicator = result
+
+        return result
+
     def draw_label(self, context, lod):
         # Skip cairo errors when drawing labels with font size 0
         # This may happen for hidden keys and keys with bad size groups.
@@ -467,7 +571,7 @@ class RectKey(Key, RectKeyCommon, DwellProgress):
         Fixme: merge with draw_label, can't do this for 0.99 because
         the Gdk.flush() workaround on the nexus 7 might fail.
         """
-        if not self.image_filenames:
+        if not self.image_filenames or not self.show_image:
             return
 
         log_rect = self.get_label_rect()
@@ -576,7 +680,7 @@ class RectKey(Key, RectKeyCommon, DwellProgress):
         context.clip()
 
         context.push_group_with_content(cairo.CONTENT_ALPHA)
-        self.build_rect_path(context, rect)
+        self._build_canvas_path(context, rect)
         context.set_source_rgba(0.0, 0.0, 0.0, 1.0)
         context.fill()
         shape = context.pop_group()
@@ -592,21 +696,32 @@ class RectKey(Key, RectKeyCommon, DwellProgress):
         # cut out the key area, the key may be transparent
         context.set_operator(cairo.OPERATOR_CLEAR)
         context.set_source_rgba(0.0, 0.0, 0.0, 1.0)
-        self.build_rect_path(context, rect)
+        self._build_canvas_path(context, rect)
         context.fill()
 
         context.restore()
 
         return surface
 
-    def get_curved_rect_params(self, rect, r_pct):
-        w, h = rect.get_size()
-        r = min(w, h) * min(r_pct / 100.0, 0.5) # full range at 50%
-        k = (r-1) * r_pct/200.0 # position of control points for circular curves
-        return r, k
+    def _build_canvas_path(self, cr, rect = None, path = None):
+        """ Build cairo path of the key geometry. """
+        if self.geometry:
+            if not path:
+                path = self.get_canvas_path()
+            self._build_complex_path(cr, path)
+        else:
+            if not rect:
+                rect = self.get_canvas_rect()
+            self._build_rect_path(cr, rect)
 
-    def build_rect_path(self, context, rect, radius_scale = 1.0):
-        roundness = config.theme_settings.roundrect_radius * radius_scale
+    def _build_complex_path(self, cr, path):
+        roundness = config.theme_settings.roundrect_radius
+        chamfer_size = self.get_chamfer_size()
+        chamfer_size = self.context.scale_log_to_canvas_y(chamfer_size)
+        rounded_path(cr, path, roundness, chamfer_size)
+
+    def _build_rect_path(self, context, rect):
+        roundness = config.theme_settings.roundrect_radius
         if roundness:
             roundrect_curve(context, rect, roundness)
         else:
@@ -652,6 +767,7 @@ class RectKey(Key, RectKeyCommon, DwellProgress):
         return extents
 
     def calc_label_base_extents(self, label):
+        """ Calculate font-size independent extents. """
         cr = Gdk.pango_context_get()
         layout = Pango.Layout(cr)
         BASE_FONTDESCRIPTION_SIZE = 10000000
@@ -699,8 +815,13 @@ class RectKey(Key, RectKeyCommon, DwellProgress):
             filename = config.get_image_filename(image_filename)
             if filename:
                 _logger.debug("loading image '{}'".format(filename))
-                pixbuf = GdkPixbuf.Pixbuf. \
-                           new_from_file_at_size(filename, width, height)
+                try:
+                    pixbuf = GdkPixbuf.Pixbuf. \
+                             new_from_file_at_size(filename, width, height)
+                except Exception as ex: # private exception gi._glib.GError when
+                                        # librsvg2-common wasn't installed
+                    _logger.error("get_image(): " + unicode_str(ex))
+
                 if pixbuf:
                     self._requested_image_size[slot] = (int(width), int(height))
 
@@ -709,7 +830,7 @@ class RectKey(Key, RectKeyCommon, DwellProgress):
         return pixbuf
 
     def _label_iterations(self, lod):
-        stroke_gradient = config.theme_settings.key_stroke_gradient / 100.0
+        stroke_gradient = self.get_stroke_gradient()
         if lod == LOD.FULL and \
            self.get_style() != "flat" and stroke_gradient:
             root = self.get_layout_root()
@@ -788,9 +909,24 @@ class WordlistKey(RectKey):
             style = "gradient"
         return style
 
+    def get_stroke_width(self):
+        # Turn down stroke width -> Only subtly bevel the wordlist bar.
+        value = super(WordlistKey, self).get_stroke_width()
+        return min(value, 0.6)
+
+    def get_stroke_gradient(self):
+        # Turn down stroke gradient -> Only subtly bevel the wordlist bar.
+        value = super(WordlistKey, self).get_stroke_gradient()
+        return min(value, 0.3)
+
     def get_light_direction(self):
         return -0.3 * pi / 180
 
+    def draw_shadow_cached(self, context):
+        # no shadow
+        pass
+
+
 
 class FullSizeKey(WordlistKey):
     def __init__(self, id = "", border_rect = None):
@@ -813,12 +949,14 @@ class BarKey(FullSizeKey):
         self.draw_image(context, lod)
         self.draw_label(context, lod)
 
-    def draw_shadow_cached(self, context):
-        pass
-
     def can_show_label_popup(self):
         return False
 
+    def get_stroke_width(self):
+        # Turn down stroke width -> no annoying banding at
+        # what should be flat key edges.
+        return 0.0
+
 
 class WordKey(FixedFontMixin, BarKey):
     def __init__(self, id="", border_rect = None):
