@@ -3,10 +3,20 @@
 from __future__ import division, print_function, unicode_literals
 
 import sys
+import time
+import weakref
 import gc
 from contextlib import contextmanager
 
-from gi.repository import Gtk, Gdk, Atspi
+from gi.repository import Gtk, Gdk
+
+import logging
+_logger = logging.getLogger(__name__)
+
+try:
+    from gi.repository import Atspi
+except ImportError as e:
+    _logger.warning("Atspi typelib missing, at-spi key-synth unavailable")
 
 from Onboard.KeyGtk          import *
 from Onboard                 import KeyCommon
@@ -16,11 +26,11 @@ from Onboard.Sound           import Sound
 from Onboard.ClickSimulator  import ClickSimulator, \
                                     CSButtonMapper, CSFloatingSlave
 from Onboard.Scanner         import Scanner
-from Onboard.utils           import Timer, Modifiers, parse_key_combination, \
+from Onboard.utils           import Timer, Modifiers, LABEL_MODIFIERS, \
+                                    parse_key_combination, \
                                     unicode_str
 from Onboard.definitions     import InputEventSourceEnum
 from Onboard.WordSuggestions import WordSuggestions
-from Onboard.AutoShow        import AutoShow
 from Onboard.canonical_equivalents import *
 
 try:
@@ -33,11 +43,6 @@ from Onboard.Config import Config
 config = Config()
 ########################
 
-### Logging ###
-import logging
-_logger = logging.getLogger("Keyboard")
-###############
-
 
 # enum of event types for key press/release
 class EventType:
@@ -114,7 +119,23 @@ class UnpressTimers:
             self._keyboard.on_key_unpressed(key)
 
 
-class KeySynthVirtkey(object):
+class KeySynth(object):
+
+    _inter_key_stroke_delay = 0.0  # delay between key-strokes in seconds
+
+    def set_inter_key_stroke_delay(self, delay):
+        """
+        Set delay between multiple key-strokes.
+        Firefox and Thunderbird may need this to not lose key-strokes.
+        """
+        self._inter_key_stroke_delay = delay
+
+    def _pause(self):
+        if self._inter_key_stroke_delay:
+            time.sleep(self._inter_key_stroke_delay)
+
+
+class KeySynthVirtkey(KeySynth):
     """ Synthesize key strokes with python-virtkey """
 
     def __init__(self, vk):
@@ -124,16 +145,19 @@ class KeySynthVirtkey(object):
         self._vk = None
 
     def press_unicode(self, char):
-        if sys.version_info.major == 2:
-            char = unicode_str(char)[0]
-        code_point = ord(char)
-        self._vk.press_unicode(code_point)
+        if self._vk:
+            if sys.version_info.major == 2:
+                char = unicode_str(char)[0]
+            code_point = ord(char)
+            self._vk.press_unicode(code_point)
 
     def release_unicode(self, char):
-        if sys.version_info.major == 2:
-            char = unicode_str(char)[0]
-        code_point = ord(char)
-        self._vk.release_unicode(code_point)
+        if self._vk:
+            if sys.version_info.major == 2:
+                char = unicode_str(char)[0]
+            code_point = ord(char)
+            self._vk.release_unicode(code_point)
+            self._pause()
 
     def press_keysym(self, keysym):
         if self._vk:
@@ -142,6 +166,7 @@ class KeySynthVirtkey(object):
     def release_keysym(self, keysym):
         if self._vk:
             self._vk.release_keysym(keysym)
+        self._pause()
 
     def press_keycode(self, keycode):
         if self._vk:
@@ -150,6 +175,7 @@ class KeySynthVirtkey(object):
     def release_keycode(self, keycode):
         if self._vk:
             self._vk.release_keycode(keycode)
+        self._pause()
 
     def lock_mod(self, mod):
         if self._vk:
@@ -169,14 +195,14 @@ class KeySynthVirtkey(object):
             for ch in keystr:
                 if ch == "\b":   # backspace?
                     keysym = get_keysym_from_name("backspace")
-                    self.press_keysym  (keysym)
+                    self.press_keysym(keysym)
                     self.release_keysym(keysym)
 
                 elif ch == "\n":
                     # press_unicode("\n") fails in gedit.
                     # -> explicitely send the key symbol instead
                     keysym = get_keysym_from_name("return")
-                    self.press_keysym  (keysym)
+                    self.press_keysym(keysym)
                     self.release_keysym(keysym)
                 else:             # any other printable keys
                     self.press_unicode(ch)
@@ -198,13 +224,115 @@ class KeySynthAtspi(KeySynthVirtkey):
         super(KeySynthAtspi, self).__init__(vk)
 
     def press_key_string(self, string):
+        if not "Atspi" in globals():
+            return
         Atspi.generate_keyboard_event(0, string, Atspi.KeySynthType.STRING)
 
     def press_keycode(self, keycode):
+        if not "Atspi" in globals():
+            return
         Atspi.generate_keyboard_event(keycode, "", Atspi.KeySynthType.PRESS)
 
     def release_keycode(self, keycode):
+        if not "Atspi" in globals():
+            return
         Atspi.generate_keyboard_event(keycode, "", Atspi.KeySynthType.RELEASE)
+        self._pause()
+
+
+class TextChanger():
+    """
+    Insert and delete text by all means available to Onboard.
+    - KeySynthVirtkey
+    - KeySynthAtspi (not used by default)
+    - Direct insertion/deletion via AtspiTextContext
+    """
+
+    def __init__(self, keyboard, vk):
+        self._keyboard = weakref.ref(keyboard)
+        self._key_synth_virtkey = KeySynthVirtkey(vk)
+        self._key_synth_atspi = KeySynthAtspi(vk)
+
+        if config.keyboard.key_synth: # == KeySynth.ATSPI:
+            self._text_changer = self._key_synth_atspi
+        else: # if config.keyboard.key_synth == KeySynth.VIRTKEY:
+            self._text_changer = self._key_synth_virtkey
+
+    def cleanup(self):
+        # Somehow keyboard objects don't get released
+        # when switching layouts, there are still
+        # excess references/memory leaks somewhere.
+        # We need to manually release virtkey references or
+        # Xlib runs out of client connections after a couple
+        # dozen layout switches.
+        if self._key_synth_virtkey:
+            self._key_synth_virtkey.cleanup()
+            self._key_synth_virtkey = None
+        if self._key_synth_atspi:
+            self._key_synth_atspi.cleanup()
+            self._key_synth_atspi = None
+
+    def get_keyboard(self):
+        return self._keyboard()
+
+    # KeySynth interface
+    def press_unicode(self, char):
+        self._text_changer.press_unicode(char)
+
+    def release_unicode(self, char):
+        self._text_changer.release_unicode(char)
+
+    def press_key_string(self, string):
+        self._text_changer.press_key_string(string)
+
+    def press_keycode(self, keycode):
+        self._text_changer.press_keycode(keycode)
+
+    def release_keycode(self, keycode):
+        self._text_changer.release_keycode(keycode)
+
+    def press_keysym(self, keysym):
+        self._text_changer.press_keysym(keysym)
+
+    def release_keysym(self, keysym):
+        self._text_changer.release_keysym(keysym)
+
+    def lock_mod(self, mod):
+        self._text_changer.lock_mod(mod)
+
+    def unlock_mod(self, mod):
+        self._text_changer.unlock_mod(mod)
+
+    # Higher-level functions
+    def press_keysyms(self, key_name, count = 1):
+        """
+        Generate any number of full key-strokes for the given named key symbol.
+        """
+        keysym = get_keysym_from_name(key_name)
+        for i in range(count):
+            self.press_keysym  (keysym)
+            self.release_keysym(keysym)
+
+    def insert_string_at_caret(self, text):
+        """
+        Send key presses for all characters in a unicode string
+        and keep track of the changes in text_context.
+        """
+        text_context = self.get_keyboard().text_context
+        if text_context.can_insert_text():
+            text = text.replace("\\n", "\n")
+            text_context.insert_text_at_caret(text)
+        else:
+            self._text_changer.press_key_string(text)
+
+    def delete_at_caret(self):
+        keyboard = self.get_keyboard()
+        text_context = keyboard.text_context
+        if text_context.can_insert_text():
+            text_context.delete_text_before_caret()
+        else:
+            with keyboard.suppress_modifiers():
+                self.press_keysyms("backspace")
 
 
 class Keyboard(WordSuggestions):
@@ -215,7 +343,7 @@ class Keyboard(WordSuggestions):
     _layer_locked = False
     _last_alt_key = None
     _alt_locked   = False
-    _key_synth    = None
+    _text_changer    = None
     _click_sim    = None
 
 ### Properties ###
@@ -238,24 +366,27 @@ class Keyboard(WordSuggestions):
                    if self.mods[mask])  # bit mask of current modifiers
 
     @contextmanager
-    def suppress_modifiers(self):
+    def suppress_modifiers(self, modifiers = LABEL_MODIFIERS):
         """ Turn modifiers off temporarily. May be nested. """
-        self._push_and_clear_modifiers()
+        self._push_and_clear_modifiers(modifiers)
         yield None
         self._pop_and_restore_modifiers()
 
-    def _push_and_clear_modifiers(self):
-        self._suppress_modifiers_stack.append(self._mods.copy())
-        for mod, nkeys in self._mods.items():
+    def _push_and_clear_modifiers(self, modifiers):
+        mods = {mod : key for mod, key in self._mods.items()
+                if mod & modifiers}
+        self._suppress_modifiers_stack.append(mods)
+        for mod, nkeys in mods.items():
             if nkeys:
                 self._mods[mod] = 0
-                self._key_synth.unlock_mod(mod)
+                self._text_changer.unlock_mod(mod)
 
     def _pop_and_restore_modifiers(self):
-        self._mods = self._suppress_modifiers_stack.pop()
-        for mod, nkeys in self._mods.items():
+        mods = self._suppress_modifiers_stack.pop()
+        for mod, nkeys in mods.items():
             if nkeys:
-                self._key_synth.lock_mod(mod)
+                self._mods[mod] = nkeys
+                self._text_changer.lock_mod(mod)
 
     # currently active layer
     def _get_active_layer_index(self):
@@ -293,9 +424,10 @@ class Keyboard(WordSuggestions):
 
 ##################
 
-    def __init__(self):
+    def __init__(self, application):
         WordSuggestions.__init__(self)
 
+        self._application = weakref.ref(application)
         self._pressed_key = None
         self._last_typing_time = 0
         self._suppress_modifiers_stack = []
@@ -310,7 +442,7 @@ class Keyboard(WordSuggestions):
         self._unpress_timers = UnpressTimers(self)
         self._touch_feedback = TouchFeedback()
 
-        self._key_synth = None
+        self._text_changer = None
         self._key_synth_virtkey = None
         self._key_synth_atspi = None
 
@@ -327,6 +459,9 @@ class Keyboard(WordSuggestions):
         self._non_modifier_released = False
         self._disabled_keys = None
 
+    def get_application(self):
+        return self._application()
+
     def register_view(self, layout_view):
         self._layout_views.append(layout_view)
 
@@ -338,6 +473,10 @@ class Keyboard(WordSuggestions):
         for view in self._layout_views:
             view.set_visible(visible)
 
+    def toggle_views_visible(self):
+        for view in self._layout_views:
+            view.toggle_visible()
+
     def redraw(self, keys = None, invalidate = True):
         for view in self._layout_views:
             view.redraw(keys, invalidate)
@@ -370,7 +509,7 @@ class Keyboard(WordSuggestions):
         if event_source == InputEventSourceEnum.XINPUT:
             # XInput click simulator
             # Recommended, but requires the XInput event source.
-            clicksim = CSFloatingSlave(self)  
+            clicksim = CSFloatingSlave(self)
 
             # Fall back to button mapper if XInput 2.2 is unavaliable
             if not clicksim.is_valid():
@@ -383,7 +522,7 @@ class Keyboard(WordSuggestions):
             # Button mapper
             # Works with any event source, but may fail on touch-screens.
             clicksim = CSButtonMapper()
-                               
+
         if self._click_sim:
             self._click_sim.cleanup()
         self._click_sim = clicksim
@@ -412,7 +551,7 @@ class Keyboard(WordSuggestions):
 
         WordSuggestions.on_layout_loaded(self)
 
-        # Update Onboard to show the initial modifiers
+        # show the currently active modifiers
         keymap = Gdk.Keymap.get_default()
         if keymap:
             mod_mask = keymap.get_modifier_state()
@@ -423,14 +562,8 @@ class Keyboard(WordSuggestions):
         self.invalidate_ui()
         self.commit_ui_updates()
 
-    def init_key_synth(self, vk):
-        self._key_synth_virtkey = KeySynthVirtkey(vk)
-        self._key_synth_atspi = KeySynthAtspi(vk)
-
-        if config.keyboard.key_synth: # == KeySynth.ATSPI:
-            self._key_synth = self._key_synth_atspi
-        else: # if config.keyboard.key_synth == KeySynth.VIRTKEY:
-            self._key_synth = self._key_synth_virtkey
+    def set_virtkey(self, vk):
+        self._text_changer = TextChanger(self, vk)
 
     def _connect_button_controllers(self):
         """ connect button controllers to button keys """
@@ -441,8 +574,8 @@ class Keyboard(WordSuggestions):
                    [BCMiddleClick, BCSingleClick, BCSecondaryClick,
                     BCDoubleClick, BCDragClick, BCHoverClick,
                     BCHide, BCShowClick, BCMove, BCPreferences, BCQuit,
+                    BCExpandCorrections, BCPauseLearning, BCLanguage,
                     BCStealthMode, BCAutoLearn, BCAutoPunctuation, BCInputline,
-                    BCExpandCorrections, BCLanguage,
                    ]
                 }
         for key in self.layout.iter_global_keys():
@@ -528,11 +661,9 @@ class Keyboard(WordSuggestions):
                time.time() - self._last_typing_time <= 0.3
 
     def _is_text_insertion_key(self, key):
-        """ Does key actually insert any characters (not navigation key)? """
-        return not key.is_modifier() and \
-               not key.type in [KeyCommon.KEYSYM_TYPE, # Fx
-                                KeyCommon.KEYPRESS_NAME_TYPE, # cursor
-                               ]
+        """ Does key actually insert any characters (not a navigation key)? """
+        return key and \
+               key.is_text_changing()
 
     def key_down(self, key, view = None, sequence = None, action = True):
         """
@@ -606,7 +737,6 @@ class Keyboard(WordSuggestions):
                 if update or \
                    key.type == KeyCommon.BUTTON_TYPE:
                     self.invalidate_context_ui()
-                    self.commit_ui_updates()
 
             # Is the key still nothing but pressed?
             extend_pressed_state = extend_pressed_state and \
@@ -643,6 +773,9 @@ class Keyboard(WordSuggestions):
             self._pressed_key = None
             gc.enable()
 
+        # Process pending UI updates
+        self.commit_ui_updates()
+
     def key_long_press(self, key, view = None, button = 1):
         """ Long press of one of Onboard's key representations. """
         long_pressed = False
@@ -761,11 +894,15 @@ class Keyboard(WordSuggestions):
 
             # Alt is special because it activates the window manager's move mode.
             if modifier != 8: # not Alt?
-                self._key_synth.lock_mod(modifier)
+                self._text_changer.lock_mod(modifier)
+
+            # Update word suggestions on shift press.
+            self.invalidate_context_ui()
 
             key.activated = True  # modifiers set -> can't undo press anymore
 
-    def send_key_up(self, key, view = None, button = 1, event_type = EventType.CLICK):
+    def send_key_up(self, key, view = None, button = 1,
+                    event_type = EventType.CLICK):
         if self.is_key_disabled(key):
             _logger.debug("send_key_up: "
                           "rejecting blacklisted key action for '{}'" \
@@ -786,8 +923,8 @@ class Keyboard(WordSuggestions):
                 self.maybe_send_alt_press_for_key(key, view, button, event_type)
 
                 if key_type == KeyCommon.CHAR_TYPE:
-                    # allow to use Atspi for char keys
-                    self.press_key_string(key.code)
+                    # allow to use AT-SPI direct text insertion for char keys
+                    self._text_changer.insert_string_at_caret(key.code)
                 else:
                     self.send_key_press(key, view, button, event_type)
                     self.send_key_release(key, view, button, event_type)
@@ -795,20 +932,24 @@ class Keyboard(WordSuggestions):
                 self.send_key_release(key, view, button, event_type)
 
         if modifier:
-            # Decrement this before unlock_mod() to skip
-            # updating keys a second time in set_modifiers().
+            # Decrement before unlock_mod() to skip updating
+            # keys a second time in set_modifiers().
             self.mods[modifier] -= 1
 
             # Alt is special because it activates the window managers move mode.
             if modifier != 8: # not Alt?
-                self._key_synth.unlock_mod(modifier)
+                self._text_changer.unlock_mod(modifier)
+
+            # Update word suggestions on shift unlatch or release.
+            self.invalidate_context_ui()
 
         self.maybe_send_alt_release_for_key(key, view, button, event_type)
 
         # Check modifier counts for plausibility.
         # There might be a bug lurking that gets certain modifers stuck
         # with negative counts. Work around this and be verbose about it
-        # so we can fix it evetually.
+        # so we can fix it eventually.
+        # Seems fixed in 0.99, but keep the check just in case.
         for mod, nkeys in self._mods.items():
             if nkeys < 0:
                 _logger.warning("Negative count {} for modifier {}, reset." \
@@ -834,14 +975,14 @@ class Keyboard(WordSuggestions):
             self._alt_locked = True
             if self._last_alt_key:
                 self.send_key_press(self._last_alt_key, view, button, event_type)
-            self._key_synth.lock_mod(8)
+            self._text_changer.lock_mod(8)
 
     def maybe_send_alt_release(self, view, button, event_type):
         if self._alt_locked:
             self._alt_locked = False
             if self._last_alt_key:
                 self.send_key_release(self._last_alt_key, view, button, event_type)
-            self._key_synth.unlock_mod(8)
+            self._text_changer.unlock_mod(8)
 
     def send_key_press(self, key, view, button, event_type):
         """ Actually generate a fake key press """
@@ -849,17 +990,17 @@ class Keyboard(WordSuggestions):
         key_type = key.type
 
         if key_type == KeyCommon.KEYCODE_TYPE:
-            self._key_synth.press_keycode(key.code)
+            self._text_changer.press_keycode(key.code)
 
         elif key_type == KeyCommon.KEYSYM_TYPE:
-            self._key_synth.press_keysym(key.code)
+            self._text_changer.press_keysym(key.code)
 
         elif key_type == KeyCommon.CHAR_TYPE:
             if len(key.code) == 1:
-                self._key_synth.press_unicode(key.code)
+                self._text_changer.press_unicode(key.code)
 
         elif key_type == KeyCommon.KEYPRESS_NAME_TYPE:
-            self._key_synth.press_keysym(get_keysym_from_name(key.code))
+            self._text_changer.press_keysym(get_keysym_from_name(key.code))
 
         elif key_type == KeyCommon.BUTTON_TYPE:
             activated = False
@@ -881,18 +1022,18 @@ class Keyboard(WordSuggestions):
         key_type = key.type
         if key_type == KeyCommon.CHAR_TYPE:
             if len(key.code) == 1:
-                self._key_synth.release_unicode(key.code)
+                self._text_changer.release_unicode(key.code)
             else:
-                self.press_key_string(key.code)
+                self._text_changer.insert_string_at_caret(key.code)
 
         elif key_type == KeyCommon.KEYSYM_TYPE:
-            self._key_synth.release_keysym(key.code)
+            self._text_changer.release_keysym(key.code)
 
         elif key_type == KeyCommon.KEYPRESS_NAME_TYPE:
-            self._key_synth.release_keysym(get_keysym_from_name(key.code))
+            self._text_changer.release_keysym(get_keysym_from_name(key.code))
 
         elif key_type == KeyCommon.KEYCODE_TYPE:
-            self._key_synth.release_keycode(key.code);
+            self._text_changer.release_keycode(key.code);
 
         elif key_type == KeyCommon.BUTTON_TYPE:
             controller = self.button_controllers.get(key)
@@ -903,7 +1044,7 @@ class Keyboard(WordSuggestions):
             snippet_id = int(key.code)
             mlabel, mString = config.snippets.get(snippet_id, (None, None))
             if mString:
-                self.press_key_string(mString)
+                self._text_changer.insert_string_at_caret(mString)
 
             # Block dialog in xembed mode.
             # Don't allow to open multiple dialogs in force-to-top mode.
@@ -923,13 +1064,8 @@ class Keyboard(WordSuggestions):
         # release key
         self.send_key_up(key, view, button, event_type)
 
-        # Insert words on button release to avoid having the wordlist
-        # change between button press and release. This also allows for
-        # long presses to trigger a different action, e.g. menu.
-        WordSuggestions.send_key_up(self, key, button, event_type)
-
         # Don't release latched modifiers for click buttons yet,
-        # Keep them unchanged until the actual click happens
+        # keep them unchanged until the actual click happens.
         # -> allow clicks with modifiers
         if not key.is_layer_button() and \
            not (key.type == KeyCommon.BUTTON_TYPE and \
@@ -941,6 +1077,12 @@ class Keyboard(WordSuggestions):
             # undo temporary suppression of the text display
             WordSuggestions.show_input_line_on_key_release(self, key)
 
+        # Insert words on button release to avoid having the wordlist
+        # change between button press and release.
+        # Make sure latched modifiers have been released, else they will
+        # affect the whole inserted string.
+        WordSuggestions.send_key_up(self, key, button, event_type)
+
         # Send punctuation after the key press and after sticky keys have
         # been released, since this may trigger latching right shift.
         #self.send_punctuation_suffix()
@@ -1159,21 +1301,9 @@ class Keyboard(WordSuggestions):
                 behavior = StickyBehavior.from_string(value)
             except KeyError:
                 _logger.warning("Invalid sticky behavior '{}' for group '{}'" \
-                              .format(value, group))
+                                .format(value, group))
         return behavior
 
-    def press_key_string(self, text):
-        """
-        Send key presses for all characters in a unicode string
-        and keep track of the changes in text_context.
-        """
-
-        if self.text_context.can_insert_text():
-            text = text.replace("\\n", "\n")
-            self.text_context.insert_text_at_cursor(text)
-        else:
-            self._key_synth.press_key_string(text)
-
     def on_snippets_dialog_closed(self):
         self.editing_snippet = False
 
@@ -1250,16 +1380,20 @@ class Keyboard(WordSuggestions):
             # modifiers may change many key labels -> redraw everything
             self.redraw_labels(False)
 
-    def release_locked_sticky_keys(self):
+    def release_locked_sticky_keys(self, release_all = False):
         """ release locked sticky (modifier) keys """
         if len(self._locked_sticky_keys) > 0:
             for key in self._locked_sticky_keys[:]:
-                self.send_key_up(key)
-                self._locked_sticky_keys.remove(key)
-                key.active = False
-                key.locked = False
-                key.pressed = False
-                self.redraw([key])
+                # NumLock is special, keep its state on exit
+                # if not told otherwise.
+                if release_all or \
+                   not key.modifier == Modifiers.NUMLK:
+                    self.send_key_up(key)
+                    self._locked_sticky_keys.remove(key)
+                    key.active = False
+                    key.locked = False
+                    key.pressed = False
+                    self.redraw([key])
 
             # modifiers may change many key labels -> redraw everything
             self.redraw_labels(False)
@@ -1286,7 +1420,7 @@ class Keyboard(WordSuggestions):
     def invalidate_ui(self):
         """
         Update everything.
-        Relatively expensive, don't call this while typing.
+        Quite expensive, don't call this while typing.
         """
         self._invalidated_ui |= UIMask.ALL
 
@@ -1305,14 +1439,14 @@ class Keyboard(WordSuggestions):
 
     def invalidate_layout(self):
         """
-        Update everything assuming key sizes don't change.
-        Doesn't invalidate cached surfaces.
+        Recalculate item rectangles.
         """
         self._invalidated_ui |= UIMask.LAYOUT
 
     def invalidate_visible_layers(self):
         """
-        Update the layout tree when the active layer changed.
+        Update visibility of layers in the layout tree,
+        e.g. when the active layer changed.
         """
         self._invalidated_ui |= UIMask.LAYERS
 
@@ -1430,13 +1564,11 @@ class Keyboard(WordSuggestions):
         # reset still latched and locked modifier keys on exit
         self.release_latched_sticky_keys()
 
-        # NumLock is special, keep its state on exit
-        if not config.keyboard.sticky_key_release_delay:
-            for key in self._locked_sticky_keys[:]:
-                if key.modifier == Modifiers.NUMLK:
-                    self._locked_sticky_keys.remove(key)
-
-        self.release_locked_sticky_keys()
+        # NumLock is special. Keep its state on exit, except when
+        # sticky_key_release_delay is set, when we assume to be
+        # in kiosk mode and everything has to be cleaned up.
+        release_all = bool(config.keyboard.sticky_key_release_delay)
+        self.release_locked_sticky_keys(release_all)
 
         # Clear key.pressed for all keys that have already been released
         # but are still waiting for redrawing the unpressed state.
@@ -1456,17 +1588,9 @@ class Keyboard(WordSuggestions):
                               .format(key.id))
                 self.send_key_up(key)
 
-        # Somehow keyboard objects don't get released
-        # when switching layouts, there are still
-        # excess references/memory leaks somewhere.
-        # We need to manually release virtkey references or
-        # Xlib runs out of client connections after a couple
-        # dozen layout switches.
-        if self._key_synth_virtkey:
-            self._key_synth_virtkey.cleanup()
-        if self._key_synth_atspi:
-            self._key_synth_atspi.cleanup()
-        self.layout = None  # free the memory
+        if self._text_changer:
+            self._text_changer.cleanup()
+        self.layout = None
 
         if self._click_sim:
             self._click_sim.cleanup()
@@ -1772,7 +1896,9 @@ class BCQuit(ButtonController):
     id = "quit"
 
     def release(self, view, button, event_type):
-        view.emit_quit_onboard()
+        app = self.keyboard.get_application()
+        if app:
+            app.emit_quit_onboard()
 
     def update(self):
         self.set_visible(not config.xid_mode and \
@@ -1788,6 +1914,88 @@ class BCExpandCorrections(ButtonController):
         wordlist.expand_corrections(not wordlist.are_corrections_expanded())
 
 
+class BCPauseLearning(ButtonController):
+
+    id = "pause-learning"
+
+    def release(self, view, button, event_type):
+        keyboard = self.keyboard
+        key = self.key
+
+        active, locked = keyboard.step_sticky_key_state(key,
+                                                        key.active, key.locked,
+                                                        button, event_type)
+        key.active  = active
+        key.locked  = locked
+
+        value = 0
+        if active:
+            value += 1
+        if locked:
+            value += 1
+
+        begin = config.word_suggestions.get_pause_learning() == 0 and \
+                value > 0
+
+        config.word_suggestions.set_pause_learning(value)
+
+        # immediately forget changes
+        if begin:
+            keyboard.discard_changes()
+
+    def update(self):
+        co = config.word_suggestions
+        self.set_active(co.get_pause_learning() >= 1)
+        self.set_locked(co.get_pause_learning() == 2)
+
+
+class BCLanguage(ButtonController):
+
+    id = "language"
+
+    def __init__(self, keyboard, key):
+        ButtonController.__init__(self, keyboard, key)
+
+    def release(self, view, button, event_type):
+        self.set_active(not self.key.active)
+        if self.key.active:
+            self._show_menu(view, self.key, button)
+
+    def _show_menu(self, view, key, button):
+        self.keyboard.hide_touch_feedback()
+        view.show_language_menu(key, button, self._on_menu_closed)
+
+    def _on_menu_closed(self):
+        self.set_active(False)
+
+    def update(self):
+        if config.are_word_suggestions_enabled():
+            key = self.key
+            keyboard = self.keyboard
+            langdb = keyboard._languagedb
+
+            lang_id = keyboard.get_lang_id()
+            label = langdb.get_language_code(lang_id).capitalize()
+
+            if label != key.get_label() or \
+               not key.tooltip:
+                key.set_labels({0: label})
+                key.tooltip = langdb.get_language_full_name(lang_id)
+                keyboard.invalidate_ui()
+
+#---------------------------------------------------------
+# deprecated buttons
+#---------------------------------------------------------
+
+class BCInputline(ButtonController):
+
+    id = "inputline"
+
+    def release(self, view, button, event_type):
+        # hide the input line display when it is clicked
+        self.keyboard.hide_input_line()
+
+
 class BCAutoLearn(ButtonController):
 
     id = "learnmode"
@@ -1834,50 +2042,4 @@ class BCStealthMode(ButtonController):
         self.set_active(config.wp.stealth_mode)
 
 
-class BCInputline(ButtonController):
-
-    id = "inputline"
-
-    def release(self, view, button, event_type):
-        # hide the input line display when it is clicked
-        self.keyboard.hide_input_line()
-
-class BCLanguage(ButtonController):
-
-    id = "language"
-
-    def __init__(self, keyboard, key):
-        ButtonController.__init__(self, keyboard, key)
-        self._opened_on_long_press = False
-
-    def release(self, view, button, event_type):
-        if not self._opened_on_long_press:
-            self.set_active(not self.key.active)
-            if self.key.active:
-                self._show_menu(view, self.key, button)
-        self._opened_on_long_press = False
-
-    def long_press(self, view, button):
-        self._opened_on_long_press = True
-        self._show_menu(view, self.key, button)
-
-    def _show_menu(self, view, key, button):
-        self.keyboard.hide_touch_feedback()
-        view.show_language_menu(key, button, self._on_menu_closed)
-
-    def _on_menu_closed(self):
-        self.set_active(False)
-
-    def update(self):
-        if config.are_word_suggestions_enabled():
-            key = self.key
-            keyboard = self.keyboard
-
-            lang_id = keyboard.get_active_lang_id()
-            visible = bool(lang_id)
-            label = keyboard._languagedb.get_language_code(lang_id)
-            if key.visible != visible or label != key.get_label():
-                key.set_labels({0: label})
-                self.set_visible(visible)
-                keyboard.invalidate_ui()
 
