@@ -443,7 +443,7 @@ class LayoutBuilder:
     def _calc_frame_width(context):
         # calculate border around the layout
         canvas_border = context.scale_log_to_canvas((1, 1))
-        return 7 + min(canvas_border)
+        return config.POPUP_FRAME_WIDTH + min(canvas_border)
 
 
 class LayoutBuilderKeySequence(LayoutBuilder):
@@ -562,7 +562,7 @@ class LayoutBuilderKeySequence(LayoutBuilder):
 
         if line:
             lines.append(line)
-    
+
         return lines, ncolumns
 
 
@@ -578,6 +578,6 @@ class LayoutBuilderAlternatives(LayoutBuilderKeySequence):
             key.code  = label[0]
             key_sequence.append(key)
 
-        return LayoutBuilderKeySequence.build(source_key, color_scheme, 
+        return LayoutBuilderKeySequence.build(source_key, color_scheme,
                                               key_sequence)
 
