@@ -19,7 +19,7 @@ from Onboard.KeyCommon      import LOD
 from Onboard.TouchHandles   import TouchHandles
 from Onboard.LayoutView     import LayoutView
 from Onboard.AutoShow       import AutoShow
-from Onboard.utils          import Rect, Timer, FadeTimer, roundrect_arc
+from Onboard.utils          import Rect, Timer, FadeTimer
 from Onboard.WindowUtils    import WindowManipulator, Handle, \
                                    canvas_to_root_window_rect, \
                                    canvas_to_root_window_point, \
@@ -51,20 +51,34 @@ class AutoReleaseTimer(Timer):
     def __init__(self, keyboard):
         self._keyboard = keyboard
 
-    def start(self):
+    def start(self, visibility_change = None):
         self.stop()
         delay = config.keyboard.sticky_key_release_delay
+        if visibility_change == False:
+            hide_delay = config.keyboard.sticky_key_release_on_hide_delay
+            if hide_delay:
+                if delay:
+                    delay = min(delay, hide_delay)
+                else:
+                    delay = hide_delay
         if delay:
             Timer.start(self, delay)
 
     def on_timer(self):
+        # When sticky_key_release_delay is set, release NumLock too.
+        # We then assume Onboard is used in a kiosk setting, and
+        # everything has to be reset for the next customer.
+        release_all_keys = bool(config.keyboard.sticky_key_release_delay)
+        if release_all_keys:
+            config.word_suggestions.set_pause_learning(0)
         self._keyboard.release_latched_sticky_keys()
-        self._keyboard.release_locked_sticky_keys()
+        self._keyboard.release_locked_sticky_keys(release_all_keys)
         self._keyboard.active_layer_index = 0
         self._keyboard.invalidate_ui_no_resize()
         self._keyboard.commit_ui_updates()
         return False
 
+
 class InactivityTimer(Timer):
     """
     Waits for the inactivity delay and transitions between
@@ -362,9 +376,31 @@ class KeyboardWidget(Gtk.DrawingArea, WindowManipulator, LayoutView, TouchInput)
             self.canvas_rect = canvas_rect
 
         rect = self.canvas_rect.deflate(self.get_frame_width())
-        #keep_aspect = config.xid_mode and self.supports_alpha()
-        keep_aspect = False
-        layout.fit_inside_canvas(rect, keep_aspect)
+        keep_aspect = config.is_keep_aspect_ratio_enabled()
+
+        x_align = 0.5
+        aspect_change_range = (0, 100)
+        if keep_aspect:
+            if config.xid_mode:
+                value = config.system_defaults.get("xembed_aspect_change_range")
+                if not value is None:
+                    value = value[1:-1]
+                    begin, end = value.split(",")
+                    aspect_change_range = (float(begin), float(end))
+
+                if config.launched_by == config.LAUNCHER_UNITY_GREETER:
+                    offset = config.system_defaults.get("xembed_unity_greeter_offset_x")
+                    if not offset is None:
+                        try:
+                            offset = float(offset)
+                            rect.x += offset
+                            rect.w -= offset
+                            x_align = 0.0
+                        except ValueError: pass
+
+        layout.fit_inside_canvas(rect, keep_aspect,
+                                 aspect_change_range=aspect_change_range,
+                                 x_align=x_align, y_align=0.5)
 
         # update the aspect ratio of the main window
         self.on_layout_updated()
@@ -589,6 +625,9 @@ class KeyboardWidget(Gtk.DrawingArea, WindowManipulator, LayoutView, TouchInput)
                    self.inactivity_timer.is_enabled():
                     self.inactivity_timer.begin_transition(False)
 
+                # start/stop on-hide-release timer
+                self._auto_release_timer.start(visible)
+
             if done:
                 window.on_transition_done(visible_before, visible_later)
 
@@ -825,7 +864,7 @@ class KeyboardWidget(Gtk.DrawingArea, WindowManipulator, LayoutView, TouchInput)
             hit_handle = self.touch_handles.hit_test(point)
             self.touch_handles.set_pressed(hit_handle)
             if not hit_handle is None:
-                # handle clicked -> stop auto-show until button release
+                # handle clicked -> stop auto-hide until button release
                 self.stop_touch_handles_auto_hide()
             else:
                 # no handle clicked -> hide them now
@@ -938,7 +977,7 @@ class KeyboardWidget(Gtk.DrawingArea, WindowManipulator, LayoutView, TouchInput)
 
         else:
             if not hit_handle is None:
-                # handle hovered over -> extend their visible time
+                # handle hovered over: extend the time touch handles are visible
                 self.start_touch_handles_auto_hide()
 
             # Show/hide the input line
@@ -1174,10 +1213,6 @@ class KeyboardWidget(Gtk.DrawingArea, WindowManipulator, LayoutView, TouchInput)
             self.touch_handles.set_corner_radius(corner_radius)
             self.touch_handles.draw(context)
 
-    def emit_quit_onboard(self, data=None):
-        _logger.debug("Entered emit_quit_onboard")
-        self.get_kbd_window().emit("quit-onboard")
-
     def _overcome_initial_key_resistance(self, sequence):
         """
         Drag-select: Increase the hit area of the initial key
@@ -1210,14 +1245,14 @@ class KeyboardWidget(Gtk.DrawingArea, WindowManipulator, LayoutView, TouchInput)
     def get_frame_width(self):
         """ Width of the frame around the keyboard; canvas coordinates. """
         if config.xid_mode:
-            return 1.0
+            return config.UNDECORATED_FRAME_WIDTH
         if config.has_window_decoration():
             return 0.0
         co = self.get_kbd_window().get_orientation_config_object()
         if config.is_dock_expanded(co):
             return 0.5
         if config.window.transparent_background:
-            return 1.0
+            return 3.0
         return config.UNDECORATED_FRAME_WIDTH
 
     def get_hit_frame_width(self):
@@ -1362,12 +1397,11 @@ class KeyboardWidget(Gtk.DrawingArea, WindowManipulator, LayoutView, TouchInput)
         # turn off AT-SPI listeners to prevent D-BUS deadlocks (Quantal).
         self.keyboard.on_focusable_gui_opening()
 
-        dialog = Gtk.Dialog(_("New snippet"),
-                            self.get_toplevel(), 0,
-                            (Gtk.STOCK_CANCEL,
-                             Gtk.ResponseType.CANCEL,
-                             _("_Save snippet"),
-                             Gtk.ResponseType.OK))
+        dialog = Gtk.Dialog(title=_("New snippet"),
+                           transient_for=self.get_toplevel(),
+                           flags=0)
+        dialog.add_button(Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL)
+        dialog.add_button(_("_Save snippet"), Gtk.ResponseType.OK)
 
         # Don't hide dialog behind the keyboard in force-to-top mode.
         if config.is_force_to_top():
@@ -1379,17 +1413,17 @@ class KeyboardWidget(Gtk.DrawingArea, WindowManipulator, LayoutView, TouchInput)
                       spacing=12, border_width=5)
         dialog.get_content_area().add(box)
 
-        msg = Gtk.Label(_("Enter a new snippet for this button:"),
+        msg = Gtk.Label(label=_("Enter a new snippet for this button:"),
                         xalign=0.0)
         box.add(msg)
 
         label_entry = Gtk.Entry(hexpand=True)
         text_entry  = Gtk.Entry(hexpand=True, activates_default = True)
-        label_label = Gtk.Label(_("_Button label:"),
+        label_label = Gtk.Label(label=_("_Button label:"),
                                 xalign=0.0,
                                 use_underline=True,
                                 mnemonic_widget=label_entry)
-        text_label  = Gtk.Label(_("S_nippet:"),
+        text_label  = Gtk.Label(label=_("S_nippet:"),
                                 xalign=0.0,
                                 use_underline=True,
                                 mnemonic_widget=text_entry)
@@ -1449,13 +1483,18 @@ class LanguageMenu:
         self._closure = closure
         self._keyboard.on_focusable_gui_opening()
 
+        keyboard = self._keyboard
+        languagedb = keyboard._languagedb
+
+        active_lang_id = keyboard.get_active_lang_id()
+        system_lang_id = keyboard.get_system_default_lang_id()
+
+        lang_ids = set(languagedb.get_language_ids())
+        if system_lang_id in lang_ids:
+            lang_ids.remove(system_lang_id)
+
         max_mru_languages = config.typing_assistance.max_recent_languages
         all_mru_lang_ids = config.typing_assistance.recent_languages
-
-        languagedb = self._keyboard._languagedb
-        lang_ids = set(languagedb.get_language_ids())
-        active_lang_id = self._keyboard.get_active_lang_id()
-
         mru_lang_ids    = [id for id in all_mru_lang_ids if id in lang_ids] \
                           [:max_mru_languages]
         other_lang_ids  = set(lang_ids).difference(mru_lang_ids)
@@ -1465,6 +1504,9 @@ class LanguageMenu:
             if name:
                 other_langs.append((name, lang_id))
 
+        def add_system_indicator(name):
+            return name + _(" - System Language")
+
         # language sub menu
         lang_menu = Gtk.Menu()
         for name, lang_id in sorted(other_langs):
@@ -1475,7 +1517,9 @@ class LanguageMenu:
         # popup menu
         menu = Gtk.Menu()
 
-        item = Gtk.CheckMenuItem.new_with_mnemonic(_("_System Language"))
+        active_lang_id = keyboard.get_active_lang_id()
+        name = languagedb.get_language_full_name(system_lang_id)
+        item = Gtk.CheckMenuItem.new_with_mnemonic(name)
         item.set_draw_as_radio(True)
         item.set_active(not active_lang_id)
         item.connect("activate", self._on_language_activated, "")
@@ -1526,11 +1570,17 @@ class LanguageMenu:
         return x, r.bottom(), True
 
     def _on_language_activated(self, menu, lang_id):
-        self._keyboard.set_active_lang_id(lang_id)
+        system_lang_id = self._keyboard.get_system_default_lang_id()
+        if lang_id == system_lang_id:
+            lang_id = ""
+        self._set_active_lang_id(lang_id)
 
     def _on_other_language_activated(self, menu, lang_id):
         if lang_id:  # empty string = system default
             self._set_mru_lang_id(lang_id)
+        self._set_active_lang_id(lang_id)
+
+    def _set_active_lang_id(self, lang_id):
         self._keyboard.set_active_lang_id(lang_id)
 
     def _set_mru_lang_id(self, lang_id):
