@@ -13,7 +13,7 @@ config = Config()
 
 class KeyContext(object):
     """
-    Transforms logical coordinates into canvas coordinates and vice versa.
+    Transforms logical coordinates to canvas coordinates and vice versa.
     """
     def __init__(self):
         # logical rectangle as defined by the keyboard layout,
@@ -74,10 +74,14 @@ class KeyContext(object):
                     self.scale_canvas_to_log_y(rect.h))
 
     def canvas_to_log_x(self, x):
-        return x * self.log_rect.w / self.canvas_rect.w + self.log_rect.x
+        canvas_rect = self.canvas_rect
+        log_rect = self.log_rect
+        return (x - canvas_rect.x) * log_rect.w / canvas_rect.w + log_rect.x
 
     def canvas_to_log_y(self, y):
-        return y * self.log_rect.h / self.canvas_rect.h + self.log_rect.y
+        canvas_rect = self.canvas_rect
+        log_rect = self.log_rect
+        return (y - canvas_rect.y) * log_rect.h / canvas_rect.h + log_rect.y
 
 
     def scale_canvas_to_log_x(self, x):
@@ -86,8 +90,17 @@ class KeyContext(object):
     def scale_canvas_to_log_y(self, y):
         return y * self.log_rect.h / self.canvas_rect.h
 
+    def log_to_canvas_path(self, path):
+        result = path.copy()
+        log_to_canvas_x = self.log_to_canvas_x
+        log_to_canvas_y = self.log_to_canvas_y
+        for op, coords in result.segments:
+            for i in range(0, len(coords), 2):
+                coords[i]   = log_to_canvas_x(coords[i])
+                coords[i+1] = log_to_canvas_y(coords[i+1])
+        return result
 
-    ##### Speed optimized overloads #####
+    ##### Speed-optimized overloads #####
 
     def log_to_canvas(self, coord):
         canvas_rect = self.canvas_rect
@@ -130,6 +143,7 @@ class LayoutRoot:
     def __init__(self, item):
         self.__dict__['_item'] = item    # item to decorate
         self.invalidate_caches()
+        self.init_chamfer_sizes()
 
     def __getattr__(self, name):
         return getattr(self._item, name)
@@ -159,9 +173,12 @@ class LayoutRoot:
         self._last_hit_args = None
         self._last_hit_key = None
 
-    def fit_inside_canvas(self, canvas_border_rect, keep_aspect = False,
-                                x_align = 0.5, y_align = 0.0):
-        self._item.fit_inside_canvas(canvas_border_rect, keep_aspect,
+    def fit_inside_canvas(self, canvas_border_rect,
+                          keep_aspect = False,
+                          aspect_change_range = (0.0, 1.1),
+                          x_align = 0.5, y_align = 0.0):
+        self._item.fit_inside_canvas(canvas_border_rect,
+                                     keep_aspect, aspect_change_range,
                                      x_align, y_align)
 
         # rects likely changed
@@ -251,10 +268,13 @@ class LayoutRoot:
         x, y = point
         hit_rects = self._get_hit_rects(active_layer)
         for x0, y0, x1, y1, k in hit_rects:
+            # Inlined test, not using Rect.is_point_within for speed.
             if x >= x0 and x < x1 and \
                y >= y0 and y < y1:
-                key = k
-                break
+                if k.geometry is None or \
+                   k.get_hit_path().is_point_within(point):
+                    key = k
+                    break
 
         self._last_hit_args = args
         self._last_hit_key = key
@@ -278,6 +298,32 @@ class LayoutRoot:
 
         return hit_rects
 
+    def init_chamfer_sizes(self):
+        chamfer_sizes = self._calc_chamfer_sizes()
+        for key in self.iter_global_keys():
+            if key.chamfer_size is None:
+                layer_id = key.get_layer()
+                chamfer_size = chamfer_sizes.get(layer_id)
+                if not chamfer_size is None:
+                    key.chamfer_size = chamfer_size
+
+    def _calc_chamfer_sizes(self):
+        chamfer_sizes = {}
+        for layer_id in [None] + self.get_layer_ids():
+            # find the most frequent key width or height of the layer
+            hist = {}
+            for key in self.iter_layer_keys(layer_id):
+                r = key.get_border_rect()
+                s = min(r.w, r.h)
+                hist[s] = hist.get(s, 0) + 1
+            most_frequent_size = \
+                max(list(zip(list(hist.values()), list(hist.keys()))))[1] \
+                if hist else None
+            chamfer_size = most_frequent_size * 0.5 \
+                if not most_frequent_size is None else None
+            chamfer_sizes[layer_id] = chamfer_size
+        return chamfer_sizes
+
 
 class LayoutItem(TreeItem):
     """ Abstract base class for layoutable items """
@@ -318,10 +364,6 @@ class LayoutItem(TreeItem):
     # parent item of sublayout roots
     sublayout_parent = None
 
-    # parsing helpers, only valid while loading a layout
-    templates = None
-    keysym_rules = None
-
     # override switching back to layer 0 on key press
     # True:  do switch to layer 0 on press
     # False: dont't
@@ -334,6 +376,9 @@ class LayoutItem(TreeItem):
     # Determines scanning order
     scan_priority = None
 
+    # parsing helpers, only valid while loading a layout
+    templates = None
+    keysym_rules = None
 
     def __init__(self):
         self.context = KeyContext()
@@ -424,8 +469,10 @@ class LayoutItem(TreeItem):
         root = self.get_layout_root()
         return root.context.scale_log_to_canvas((2.0, 2.0))
 
-    def fit_inside_canvas(self, canvas_border_rect, keep_aspect = False,
-                                x_align = 0.5, y_align = 0.0):
+    def fit_inside_canvas(self, canvas_border_rect,
+                          keep_aspect = False,
+                          aspect_change_range = (0.0, 1.1),
+                          x_align = 0.5, y_align = 0.0):
         """
         Scale item and its children to fit inside the given canvas_rect.
         """
@@ -434,9 +481,17 @@ class LayoutItem(TreeItem):
 
         # optionally maintain the aspect ratio and align the result
         if keep_aspect:
-            log_rect = self.context.log_rect
-            canvas_border_rect = log_rect.inscribe_with_aspect( \
-                                        canvas_border_rect, x_align, y_align)
+            r = self.context.log_rect
+            if r.h:
+                a0 = r.w / float(r.h)
+                a0_max = a0 * aspect_change_range[1]
+                a1 = canvas_border_rect.w / float(canvas_border_rect.h)
+                a = min(a1, a0_max)
+
+                r = Rect(0, 0, a, 1.0)
+                r = Rect(0, 0, a, 1.0)
+            canvas_border_rect = r.inscribe_with_aspect( \
+                                      canvas_border_rect, x_align, y_align)
 
         # recursively fit inside canvas
         self._fit_inside_canvas(canvas_border_rect)
@@ -519,8 +574,16 @@ class LayoutItem(TreeItem):
                 return item
             item = item.parent
 
+    def get_global_layout_root(self):
+        """ Return the root layout item """
+        item = self
+        while item:
+            if item.parent is None:
+                return item
+            item = item.parent
+
     def get_layer(self):
-        """ Return the first layer on the path from the tree root to self """
+        """ Return the first layer_id on the path from the tree root to self """
         layer_id = None
         item = self
         while item:
