@@ -15,15 +15,18 @@ from xml.dom import minidom
 
 from Onboard                 import Exceptions
 from Onboard                 import KeyCommon
-from Onboard.KeyCommon       import StickyBehavior, ImageSlot
-from Onboard.KeyGtk          import RectKey, WordlistKey, BarKey, \
-                                    WordKey, InputlineKey
+from Onboard.KeyCommon       import StickyBehavior, ImageSlot, \
+                                    KeyPath, KeyGeometry
 from Onboard.Layout          import LayoutRoot, LayoutBox, LayoutPanel
-from Onboard.utils           import hexstring_to_float, modifiers, Rect, \
+from Onboard.utils           import modifiers, Rect, \
                                     toprettyxml, Version, open_utf8, \
                                     permute_mask, LABEL_MODIFIERS, \
                                     unicode_str, XDGDirs
+
+# Layout items that can be created dynamically via the 'class' XML attribute.
 from Onboard.WordSuggestions import WordListPanel
+from Onboard.KeyGtk          import RectKey, WordlistKey, BarKey, \
+                                    WordKey, InputlineKey
 
 ### Config Singleton ###
 from Onboard.Config import Config
@@ -119,7 +122,7 @@ class LayoutLoaderSVG:
             root = LayoutPanel() # root, representing the 'keyboard' tag
             root.set_id("__root__") # id for debug prints
 
-            # Init included root with the parent items svg filename.
+            # Init included root with the parent item's svg filename.
             # -> Allows to skip specifying svg filenames in includes.
             if parent_item:
                 root.filename = parent_item.filename
@@ -367,26 +370,30 @@ class LayoutLoaderSVG:
         # get key geometry from the closest svg file
         filename = key.get_filename()
         if not filename:
-            if not attribute.get("group") == "wsbutton": 
+            if not attribute.get("group") == "wsbutton":
                 _logger.warning(_format("Ignoring key '{}'."
                                         " No svg filename defined.",
                                         key.id))
         else:
-            svg_keys = self._get_svg_keys(filename)
-            svg_key = None
-            if svg_keys:
-                # try svg_id first
+            svg_nodes = self._get_svg_keys(filename)
+            if svg_nodes:
+                # try svg_id first, if there is one
                 if key.svg_id != key.id:
-                    svg_key = svg_keys.get(key.svg_id)
+                    svg_node = svg_nodes.get(key.svg_id)
                 else:
                     # then the regular id
-                    svg_key = svg_keys.get(key.id)
+                    svg_node = svg_nodes.get(key.id)
 
-                if svg_key:
-                    r = svg_key.get_border_rect()
+                if svg_node:
+                    r, geometry = svg_node.extract_key_params()
                     key.set_initial_border_rect(r.copy())
                     key.set_border_rect(r.copy())
+                    key.geometry = geometry
                     result = key
+                else:
+                    _logger.info("Ignoring key '{}'."
+                                 " No svg object found for '{}'." \
+                                 .format(key.id, key.svg_id))
 
         return result  # ignore keys not found in an svg file
 
@@ -398,6 +405,10 @@ class LayoutLoaderSVG:
         svg_id = attributes.get("svg_id")
         key.set_id(full_id, theme_id, svg_id)
 
+        if "_" in key.get_id():
+            _logger.warning("underscore in key id '{}', please use dashes" \
+                            .format(key.get_id()))
+
         value = attributes.get("modifier")
         if value:
             try:
@@ -543,6 +554,9 @@ class LayoutLoaderSVG:
         if "popup_id" in attributes:
             key.popup_id = attributes["popup_id"]
 
+        if "chamfer_size" in attributes:
+            key.chamfer_size = float(attributes["chamfer_size"])
+
         key.color_scheme = self._color_scheme
 
     def _parse_key_labels(self, attributes, key):
@@ -637,45 +651,63 @@ class LayoutLoaderSVG:
         return labels
 
     def _get_svg_keys(self, filename):
-        svg_keys = self._svg_cache.get(filename)
-        if svg_keys is None:
-            svg_keys = self._load_svg_keys(filename)
-            self._svg_cache[filename] = svg_keys
+        svg_nodes = self._svg_cache.get(filename)
+        if svg_nodes is None:
+            svg_nodes = self._load_svg_keys(filename)
+            self._svg_cache[filename] = svg_nodes
 
-        return svg_keys
+        return svg_nodes
 
     def _load_svg_keys(self, filename):
         filename = os.path.join(self._root_layout_dir, filename)
         try:
             with open_utf8(filename) as svg_file:
                 svg_dom = minidom.parse(svg_file).documentElement
-                svg_keys = self._parse_svg(svg_dom)
+                svg_nodes = self._parse_svg(svg_dom)
+                svg_nodes = {node.id : node for node in svg_nodes}
+        except Exceptions.LayoutFileError as ex:
+            raise Exceptions.LayoutFileError(
+                "error loading '{}'".format(filename),
+                chained_exception = (ex))
+        return svg_nodes
 
-        except Exception as xxx_todo_changeme1:
-            (exception) = xxx_todo_changeme1
-            raise Exceptions.LayoutFileError(_("Error loading ")
-                + filename, chained_exception = exception)
+    def _parse_svg(self, node):
+        svg_nodes = []
+        for child in node.childNodes:
+            if child.nodeType == minidom.Node.ELEMENT_NODE:
+                tag = child.tagName
+                if tag in ("rect", "path", "g"):
+                    svg_node = SVGNode()
+                    id = child.attributes["id"].value
+                    svg_node.id = id
 
-        return svg_keys
+                    if tag == "rect":
+                        svg_node.bounds = \
+                                   Rect(float(child.attributes['x'].value),
+                                        float(child.attributes['y'].value),
+                                        float(child.attributes['width'].value),
+                                        float(child.attributes['height'].value))
 
-    def _parse_svg(self, svg_dom):
-        keys = {}
-        for rect in svg_dom.getElementsByTagName("rect"):
-            id = rect.attributes["id"].value
+                    elif tag == "path":
+                        data = child.attributes['d'].value
 
-            rect = Rect(float(rect.attributes['x'].value),
-                        float(rect.attributes['y'].value),
-                        float(rect.attributes['width'].value),
-                        float(rect.attributes['height'].value))
+                        try:
+                            svg_node.path = KeyPath.from_svg_path(data)
+                        except ValueError as ex:
+                            raise Exceptions.LayoutFileError(
+                                  "while reading geometry with id '{}'".format(id),
+                                  chained_exception = (ex))
 
-            # Use RectKey as cache for svg provided properties.
-            # This key instance doesn't enter the layout and will
-            # be discarded after the layout tree has been loaded.
-            key = RectKey(id, rect)
+                        svg_node.bounds = svg_node.path.get_bounds()
 
-            keys[id] = key
+                    elif tag == "g":  # group
+                        svg_node.children = self._parse_svg(child)
 
-        return keys
+                    svg_nodes.append(svg_node)
+
+                svg_nodes.extend(self._parse_svg(child))
+
+        return svg_nodes
 
     def find_template(self, scope_item, classinfo, ids):
         """
@@ -705,7 +737,7 @@ class LayoutLoaderSVG:
     def _get_system_keyboard_layout(self, vk):
         """ get names of the currently active layout group and variant """
 
-        if self._vk: # xkb keyboard found?
+        if vk: # xkb keyboard found?
             group = vk.get_current_group()
             names = vk.get_rules_names()
         else:
@@ -975,13 +1007,25 @@ class LayoutLoaderSVG:
 
     @staticmethod
     def _replace_basename(filename, new_basename, fallback_layer_name):
+        """
+        Doctests:
+        # Basename has to be replaced with new_basename.
+        >>> test = LayoutLoaderSVG._replace_basename
+        >>> test("/home/usr/.local/share/onboard/Base-Alpha.svg",
+        ... "NewBase","Fallback")
+        'NewBase-Alpha.svg'
+
+        # Dashes in front are allowed, but the layer name must not have any.
+        >>> test("/home/usr/.local/share/onboard/a-b-c-Alpha.svg",
+        ... "d-e-f","g-h")
+        'd-e-f-Alpha.svg'
+        """
         dir, name_ext = os.path.split(filename)
         name, ext = os.path.splitext(name_ext)
-        components = name.split("-")
-        if components:
-            basename = components[0]
-            if len(components) > 1:
-                layer = components[1]
+        if name:
+            index = name.rfind("-")
+            if index >= 0:
+                layer = name[index+1:]
             else:
                 layer = fallback_layer_name
             return "{}-{}{}".format(new_basename, layer, ext)
@@ -1003,4 +1047,28 @@ class LayoutLoaderSVG:
                     yield node
 
 
+class SVGNode:
+    """
+    Cache of SVG provided key attributes.
+    """
+    id = None             # svg_id
+    bounds = None         # logical bounding rect, aka border rect
+    path = None           # optional path for arbitrary shapes
+
+    def __init__(self):
+        self.children = []
+
+    def extract_key_params(self):
+        if self.children:
+            nodes = self.children[:2]
+        else:
+            nodes = [self]
+        bounds = nodes[0].bounds
+        paths = [node.path for node in nodes if node.path]
+        if paths:
+            geometry = KeyGeometry.from_paths(paths)
+        else:
+            geometry = None
+        return bounds, geometry
+
 
