@@ -30,7 +30,7 @@ from Onboard.LayoutLoaderSVG import LayoutLoaderSVG
 from Onboard.Appearance      import ColorScheme
 from Onboard.IconPalette     import IconPalette
 from Onboard.Exceptions      import LayoutFileError
-from Onboard.utils           import show_confirmation_dialog, CallOnce, Process, \
+from Onboard.utils           import show_confirmation_dialog, CallOnce, \
                                     unicode_str
 import Onboard.osk as osk
 
@@ -89,11 +89,11 @@ class OnboardGtk(object):
             # gss when there is already a non-embedded instance running in
             # the user session (LP: 938302).
             if config.xid_mode and \
+               config.launched_by == config.LAUNCHER_GSS and \
                not (config.gnome_a11y and \
-                    config.gnome_a11y.screen_keyboard_enabled):
-                if Process.was_launched_by("gnome-screensaver") and \
-                   not has_remote_instance:
-                    sys.exit(0)
+                    config.gnome_a11y.screen_keyboard_enabled) and \
+               not has_remote_instance:
+                sys.exit(0)
 
             # Embedded instances can't become primary instances
             if not config.xid_mode:
@@ -147,8 +147,8 @@ class OnboardGtk(object):
         sys.path.append(os.path.join(config.install_dir, 'scripts'))
 
         # Create the central keyboard model
-        self.keyboard = Keyboard()
-        
+        self.keyboard = Keyboard(self)
+
         # Create the initial keyboard widget
         # Care for toolkit independence only once there is another
         # supported one besides GTK.
@@ -172,6 +172,7 @@ class OnboardGtk(object):
             self.do_connect(self._window, "quit-onboard",
                             lambda x: self.do_quit_onboard())
 
+        #config.xid_mode = True
         self._window.application = self
         config.main_window = self._window # need this to access screen properties
 
@@ -275,6 +276,8 @@ class OnboardGtk(object):
         config.word_suggestions.show_context_line_notify_add(update_ui)
         config.word_suggestions.enabled_notify_add(lambda x: \
                                  self.keyboard.on_word_suggestions_enabled(x))
+        config.word_suggestions.auto_learn_notify_add(
+                                 update_ui_no_resize)
         config.typing_assistance.active_language_notify_add(lambda x: \
                                  self.keyboard.on_active_lang_id_changed())
         config.typing_assistance.spell_check_backend_notify_add(lambda x: \
@@ -283,12 +286,14 @@ class OnboardGtk(object):
                                  self.keyboard.on_word_suggestions_enabled(x))
         config.word_suggestions.spelling_suggestions_enabled_notify_add(lambda x: \
                                  self.keyboard.on_spell_checker_changed())
+        config.word_suggestions.wordlist_buttons_notify_add(
+                                 update_ui_no_resize)
 
         # universal access
         config.scanner.enabled_notify_add(self.keyboard._on_scanner_enabled)
 
         config.window.resize_handles_notify_add(lambda x: \
-                                    self.keyboard_widget.update_resize_handles())
+                                 self.keyboard_widget.update_resize_handles())
 
         # misc
         config.keyboard.show_click_buttons_notify_add(update_ui)
@@ -581,7 +586,7 @@ class OnboardGtk(object):
         layout = LayoutLoaderSVG().load(vk, layout_filename, color_scheme)
 
         self.keyboard.cleanup()
-        self.keyboard.init_key_synth(vk)
+        self.keyboard.set_virtkey(vk)
         self.keyboard.layout = layout
         self.keyboard.color_scheme = color_scheme
         self.keyboard.on_layout_loaded()
@@ -610,6 +615,9 @@ class OnboardGtk(object):
 
 
     # Methods concerning the application
+    def emit_quit_onboard(self):
+        self._window.emit_quit_onboard()
+
     def do_quit_onboard(self):
         _logger.debug("Entered do_quit_onboard")
         self.final_cleanup()
@@ -641,13 +649,13 @@ class OnboardGtk(object):
     @staticmethod
     def _can_show_in_current_desktop():
         """
-        When GNOME's "Typing Assistent" is enabled in GNOME Shell, Onboard 
-        starts simultaneously with the Shell's built-in screen keyboard. 
+        When GNOME's "Typing Assistent" is enabled in GNOME Shell, Onboard
+        starts simultaneously with the Shell's built-in screen keyboard.
         With GNOME Shell 3.5.4-0ubuntu2 there is no known way to choose
-        one over the other (LP: 879942). 
+        one over the other (LP: 879942).
 
-        Adding NotShowIn=GNOME; to onboard-autostart.desktop prevents it 
-        from running not only in GNOME Shell, but also in the GMOME Fallback 
+        Adding NotShowIn=GNOME; to onboard-autostart.desktop prevents it
+        from running not only in GNOME Shell, but also in the GMOME Fallback
         session, which is undesirable. Both share the same xdg-desktop name.
 
         -> Do it ourselves: optionally check for GNOME Shell and yield to the
@@ -659,7 +667,7 @@ class OnboardGtk(object):
             bus = dbus.SessionBus()
             current = os.environ.get("XDG_CURRENT_DESKTOP", "")
             names = config.options.not_show_in.split(",")
-            for name in names:                
+            for name in names:
                 if name == "GNOME":
                     if bus.name_has_owner("org.gnome.Shell"):
                         result = False
@@ -698,6 +706,10 @@ if "dbus" in globals():
         def Hide(self):
             self._keyboard.set_visible(False)
 
+        @dbus.service.method(dbus_interface=IFACE)
+        def ToggleVisible(self):
+            self._keyboard.toggle_visible()
+
         @dbus.service.method(dbus_interface=dbus.PROPERTIES_IFACE,
                              in_signature='ss', out_signature='v')
         def Get(self, iface, prop):
