@@ -24,15 +24,21 @@ class TouchHandle(object):
     corner_radius = 0     # radius of the outer corners (window edges)
 
     _size = (40, 40)
+    _fallback_size = (40, 40)
+    _hit_proximity_factor = 1.5
     _rect = None
     _scale = 1.0   # scale of handle relative to resize handles
     _handle_alpha = 0.45
     _shadow_alpha = 0.04
     _shadow_size = 8
     _shadow_offset = (0.0, 2.0)
+    _screen_dpi = 96
 
     _handle_angles = {}  # dictionary at class scope!
 
+    lock_x_axis = False
+    lock_y_axis = False
+
     def __init__(self, id):
         self.id = id
 
@@ -56,9 +62,11 @@ class TouchHandle(object):
         return min(w, h) / 2.0
 
     def get_shadow_rect(self):
-        rect = self.get_rect().inflate(self._shadow_size+1)
-        rect.w += self._shadow_offset[0]
-        rect.h += self._shadow_offset[1]
+        rect = self.get_rect()
+        if rect:
+            rect = rect.inflate(self._shadow_size+1)
+            rect.w += self._shadow_offset[0]
+            rect.h += self._shadow_offset[1]
         return rect
 
     def get_arrow_angle(self):
@@ -109,16 +117,17 @@ class TouchHandle(object):
         self._rect = Rect(x, y, w, h)
 
     def hit_test(self, point):
-        rect = self.get_rect()
+        rect   = self.get_rect().grow(self._hit_proximity_factor)
+        radius = self.get_radius() * self._hit_proximity_factor
+
         if rect and rect.is_point_within(point):
             _win = self._window.get_window()
             if _win:
                 context = _win.cairo_create()
-                self._build_handle_path(context)
+                self._build_handle_path(context, rect, radius)
                 return context.in_fill(*point)
         return False
 
-        radius = self.get_radius()
         xc, yc = rect.get_center()
         dx = xc - point[0]
         dy = yc - point[1]
@@ -198,28 +207,35 @@ class TouchHandle(object):
         radius = self.get_radius()
         xc, yc = self.get_rect().get_center()
         scale = radius / 2.0 / self._scale * 1.2
-        num_arrows = 4 if self.id == Handle.MOVE else 2
+
         angle = self.get_arrow_angle()
+        if self.id == Handle.MOVE:
+            num_arrows = 4
+            if self.lock_x_axis:
+                num_arrows -= 2
+                angle += pi * 0.5
+            if self.lock_y_axis:
+                num_arrows -= 2
+        else:
+            num_arrows = 2
         angle_step = 2.0 * pi / num_arrows
 
-        context.save()
-
         for i in range(num_arrows):
-            m = cairo.Matrix()
-            m.translate(xc, yc)
-            m.rotate(angle + i * angle_step)
-            m.scale(scale, scale)
+            context.save()
+
+            context.translate(xc, yc)
+            context.rotate(angle + i * angle_step)
+            context.scale(scale, scale)
 
             # arrow distance from center
             if self.id is Handle.MOVE:
-                m.translate(0.9, 0)
+                context.translate(0.9, 0)
             else:
-                m.translate(0.30, 0)
+                context.translate(0.30, 0)
 
-            context.set_matrix(m)
             self._draw_arrow(context)
 
-        context.restore()
+            context.restore()
 
     def _draw_arrow(self, context):
         context.move_to( 0.0, -0.5)
@@ -234,10 +250,12 @@ class TouchHandle(object):
         context.set_line_width(0)
         context.stroke()
 
-    def _build_handle_path(self, context):
-        rect = self.get_rect()
+    def _build_handle_path(self, context, rect = None, radius = None):
+        if rect is None:
+            rect = self.get_rect()
+        if radius is None:
+            radius = self.get_radius()
         xc, yc = rect.get_center()
-        radius = self.get_radius()
         corner_radius = self.corner_radius
 
         angle = self.get_arrow_angle()
@@ -291,18 +309,25 @@ class TouchHandles(object):
     rect = None
 
     def __init__(self):
-        self.handles = [TouchHandle(Handle.MOVE),
-                        TouchHandle(Handle.NORTH_WEST),
-                        TouchHandle(Handle.NORTH),
-                        TouchHandle(Handle.NORTH_EAST),
-                        TouchHandle(Handle.EAST),
-                        TouchHandle(Handle.SOUTH_EAST),
-                        TouchHandle(Handle.SOUTH),
-                        TouchHandle(Handle.SOUTH_WEST),
-                        TouchHandle(Handle.WEST)]
+        self.handles = []
+        self._handle_pool = [TouchHandle(Handle.MOVE),
+                             TouchHandle(Handle.NORTH_WEST),
+                             TouchHandle(Handle.NORTH),
+                             TouchHandle(Handle.NORTH_EAST),
+                             TouchHandle(Handle.EAST),
+                             TouchHandle(Handle.SOUTH_EAST),
+                             TouchHandle(Handle.SOUTH),
+                             TouchHandle(Handle.SOUTH_WEST),
+                             TouchHandle(Handle.WEST)]
+
+    def set_active_handles(self, handle_ids):
+        self.handles = []
+        for handle in self._handle_pool:
+            if handle.id in handle_ids:
+                self.handles.append(handle)
 
     def set_window(self, window):
-        for handle in self.handles:
+        for handle in self._handle_pool:
             handle._window = window
 
     def update_positions(self, canvas_rect):
@@ -356,3 +381,29 @@ class TouchHandles(object):
         for handle in self.handles:
             handle.corner_radius = corner_radius
 
+    def set_monitor_dimensions(self, size_px, size_mm):
+        min_monitor_mm = 50
+        target_size_mm = (5, 5)
+        min_size = TouchHandle._fallback_size
+        
+        if size_mm[0] < min_monitor_mm or \
+           size_mm[1] < min_monitor_mm:
+            w = 0
+            h = 0
+        else:
+            w = size_px[0] / size_mm[0] * target_size_mm[0]
+            h = size_px[0] / size_mm[0] * target_size_mm[0]
+        size = max(w, min_size[0]), max(h, min_size[1])
+        TouchHandle._size = size
+
+    def lock_x_axis(self, lock):
+        """ Set to False to constraint movement in x. """
+        for handle in self.handles:
+            handle.lock_x_axis = lock
+
+    def lock_y_axis(self, lock):
+        """ Set to True to constraint movement in y. """
+        for handle in self.handles:
+            handle.lock_y_axis = lock
+
+
