@@ -76,9 +76,9 @@ class InputEventSource(EventSource):
         self._device_manager = None
 
         self._master_device = None      # receives enter/leave events
-        self._master_device_id = None   # for convenience/speed only
+        self._master_device_id = None   # for convenience/performance
         self._slave_devices = None      # receive pointer and touch events
-        self._slave_device_ids = None   # for convenience/speed only
+        self._slave_device_ids = None   # for convenience/performance
 
         self._xi_grab_active = False
         self._xi_grab_events_selected = False
@@ -106,7 +106,7 @@ class InputEventSource(EventSource):
 
     def grab_xi_pointer(self, active):
         """
-        Tell the xi event source a drag operation has started/ended
+        Tell the xi event source a drag operation has started (ended)
         and we want to receive events of the whole screen.
         """
         self._xi_grab_active = active
@@ -376,6 +376,8 @@ class InputEventSource(EventSource):
 
         elif event_type == XIEventType.ButtonRelease:
             self._on_button_release_event(self, event)
+
+            # Notify CSButtonMapper, end remapped click.
             if not self._xi_event_handled:
                 EventSource.emit(self, "button-release", event)
 
@@ -412,7 +414,7 @@ class TouchInput(InputEventSource):
     GESTURE_DETECTION_SPAN = 100 # [ms] until two finger tap&drag is detected
     GESTURE_DELAY_PAUSE = 3000   # [ms] Suspend delayed sequence begin for this
                                  # amount of time after the last key press.
-    delay_sequence_begin = True  # No delivery, i.e. no key-presses after
+    DELAY_SEQUENCE_BEGIN = True  # No delivery, i.e. no key-presses after
                                  # gesture detection, but delays press-down.
 
     def __init__(self):
@@ -457,8 +459,8 @@ class TouchInput(InputEventSource):
                 return
 
         # - Ignore double clicks (GDK_2BUTTON_PRESS),
-        #   we're handling them ourselves.
-        # - no mouse wheel buttons
+        #   we're handling those ourselves.
+        # - Ignore mouse wheel button events
         if event.type == Gdk.EventType.BUTTON_PRESS and \
            1 <= event.button <= 3:
             sequence = InputSequence()
@@ -551,9 +553,10 @@ class TouchInput(InputEventSource):
             if not self._gesture_detected:
                 if first_sequence and \
                    self._multi_touch_enabled and \
-                   self.delay_sequence_begin and \
+                   self.DELAY_SEQUENCE_BEGIN and \
                    sequence.time - self._last_sequence_time > \
-                                   self.GESTURE_DELAY_PAUSE:
+                                   self.GESTURE_DELAY_PAUSE and \
+                   self.can_delay_sequence_begin(sequence): # ask Keyboard
                     # Delay the first tap; we may have to stop it
                     # from reaching the keyboard.
                     self._gesture_timer.start(self.GESTURE_DETECTION_SPAN / 1000.0,
@@ -566,6 +569,10 @@ class TouchInput(InputEventSource):
 
         self._last_sequence_time = sequence.time
 
+    def can_delay_sequence_begin(self, sequence):
+        """ Overloaded in LayoutView to veto delay for move buttons. """
+        return True
+        
     def on_delayed_sequence_begin(self, sequence, point):
         if not self._gesture_detected: # work around race condition
             sequence.point = point # return to the original begin point
@@ -689,7 +696,7 @@ class TouchInput(InputEventSource):
         sequence = self._get_redir_sequence(sequence)
 
         # Make sure has_input_sequences() returns False inside of func().
-        # The keyboard needs this to detect the end of input.
+        # Class Keyboard needs this to detect the end of input.
         if sequence.id in self._input_sequences:
             del self._input_sequences[sequence.id]
 
@@ -717,23 +724,23 @@ class TouchInput(InputEventSource):
 class InputSequence:
     """
     State of a single click- or touch sequence.
-    On a multi-touch capable touch screen, any number of
+    On a multi-touch capable touch screen any number of
     InputSequences may be in flight simultaneously.
     """
     id          = None  # sequence id, POINTER_SEQUENCE for mouse events
     point       = None  # (x, y)
     root_point  = None  # (x, y)
-    button      = None
+    button      = None  # GDK button number, 1 for touch
     event_type  = None  # Keyboard.EventType
     state       = None  # GDK state mask (Gdk.ModifierType)
     time        = None  # event time
-    update_time = None  # redundant, only used in _discard_stuck_input_sequences
+    update_time = None  # redundant, only used by _discard_stuck_input_sequences
 
     primary     = False # Only primary sequences may move/resize windows.
     delivered   = False # Sent to listeners (keyboard views)?
 
-    active_key         = None  # Currently pressed key for this sequence.
-    initial_active_key = None  # Initial pressed key for this sequence.
+    active_key         = None  # Onboard key currently pressed by this sequence.
+    initial_active_key = None  # First Onboard key pressed by this sequence.
     cancel_key_action  = False # Cancel key action, e.g. due to long press.
 
     def init_from_button_event(self, event):
@@ -772,5 +779,3 @@ class InputSequence:
         return "{}({})".format(type(self).__name__,
                                repr(self.id))
 
-
-
