@@ -206,7 +206,14 @@ osk_devices_init (OskDevices *dev, PyObject *args, PyObject *kwds)
     int major = 2;
     int minor = 2;
 
-    dev->dpy = GDK_DISPLAY_XDISPLAY (gdk_display_get_default ());
+    /* set display before anything else! */
+    GdkDisplay* display = gdk_display_get_default ();
+    if (!GDK_IS_X11_DISPLAY (display)) // Wayland, MIR?
+    {
+        PyErr_SetString (OSK_EXCEPTION, "not an X display");
+        return -1;
+    }
+    dev->dpy = GDK_DISPLAY_XDISPLAY (display);
     memset(dev->button_states, 0, sizeof(dev->button_states));
 
     if (!XQueryExtension (dev->dpy, "XInputExtension",
@@ -1137,12 +1144,13 @@ osk_devices_grab_device (PyObject *self, PyObject *args)
 {
     OskDevices       *dev = (OskDevices *) self;
     int               id;
-    Window            win;
+    int               _win;
 
-    if (!PyArg_ParseTuple (args, "ii", &id, &win))
+    if (!PyArg_ParseTuple (args, "ii", &id, &_win))
         return NULL;
 
-    if (win == 0)
+    Window win = (Window)_win;
+    if (!win)
         win = DefaultRootWindow (dev->dpy);
 
     unsigned char mask[1] = {0};
@@ -1210,10 +1218,13 @@ osk_devices_select_events (PyObject *self, PyObject *args)
     unsigned char mask[4] = { 0, 0, 0, 0};
     int           device_id;
     unsigned long event_mask;
-    Window        win = DefaultRootWindow (dev->dpy);
+    int           _win;
 
-    if (!PyArg_ParseTuple (args, "iil", &win, &device_id, &event_mask))
+    if (!PyArg_ParseTuple (args, "iil", &_win, &device_id, &event_mask))
         return NULL;
+
+    Window win = (Window) _win;
+
     if (dev->event_handler)
     {
         int i;
@@ -1248,11 +1259,13 @@ osk_devices_unselect_events (PyObject *self, PyObject *args)
     OskDevices   *dev = (OskDevices *) self;
     unsigned char mask[1] = { 0 };
     int           device_id;
-    Window        win = 0;
+    int           _win;
 
-    if (!PyArg_ParseTuple (args, "ii", &win, &device_id))
+    if (!PyArg_ParseTuple (args, "ii", &_win, &device_id))
         return NULL;
 
+    Window win = (Window) _win;
+
     if (dev->event_handler)
     {
         if (osk_devices_select (dev, win, device_id, mask, sizeof (mask)) < 0)
