@@ -32,6 +32,7 @@ class _BaseModel:
 
     modified = False
     load_error = False
+    load_error_msg = ""
 
     def learn_tokens(self, tokens, allow_new_words=True):
         """ Extract n-grams from tokens and count them. """
@@ -52,6 +53,7 @@ class _BaseModel:
         [['word1'], ['word1', 'word2'], ['word2'], ['<s>'], ['<s>', 'word3'], ['word3']]
         """
         token_sections = []
+
         # Don't let <unk> enter the model.
         # Split the token stream into sections between <unk>s.
         unk_sections = split_tokens(tokens, "<unk>")
@@ -82,12 +84,39 @@ class _BaseModel:
             totals[len(ng[0])-1] += ng[1]
         return counts, totals
 
+    def copy(self, model):
+        """
+        Copy contents of self to model. The order of the destination
+        stays unchanged.
+        """
+        if hasattr(self, "smoothing"): # not for UnigramModel
+            model.smoothing = self.smoothing
+
+        for it in self.iter_ngrams():
+            ngram = it[0]
+            count = it[1]
+            model.count_ngram(ngram, count)
+
+        return model
+
     def prune(self, prune_counts):
         """
         Return a copy of self with all ngrams removed whose
         count is less or equal to <prune_count>.
+
+        prune_count==-1  # prune all frequencies
+        prune_count=0    # prune nothing
+        prune_count>0    # prune frequencies below or equal prune_count
         """
-        model = self.__class__(self.order)
+        # drop order for to be emptied n-gram levels
+        order = self.order
+        for prune_count in reversed(prune_counts):
+            if prune_count != -1:
+                break
+            order -= 1
+
+        order = max(order, 2)
+        model = self.__class__(order)
 
         if hasattr(self, "smoothing"): # not for UnigramModel
             model.smoothing = self.smoothing
@@ -100,13 +129,14 @@ class _BaseModel:
             k = min(len(prune_counts), level) - 1
             prune_count = prune_counts[k]
 
-            if count > prune_count:
+            if count > prune_count and  prune_count != -1:
                 model.count_ngram(ngram, count)
 
         return model
 
     def load(self, filename):
         self.load_error = False
+        self.load_error_msg = ""
         self.modified = False
         try:
             super(_BaseModel, self).load(filename)
@@ -116,7 +146,10 @@ class _BaseModel:
 
 
 class LanguageModel(_BaseModel, lm.LanguageModel):
-    """ Keep this to access the class constants. """
+    """
+    Abstract class representing the base class of all models.
+    Keep this for access to class constants.
+    """
     def __init__(self):
         raise NotImplementedError()
 
@@ -138,6 +171,8 @@ class CachedDynamicModel(_BaseModel, lm.CachedDynamicModel):
 
 def split_tokens(tokens, separator, keep_separator = False):
     """
+    Split list of tokens at separator token.
+
     Doctests:
     # excluding separator
     >>> split_tokens(["<unk>", "word1", "word2", "word3"], "<unk>")
@@ -242,35 +277,53 @@ def split_sentences(text, disambiguate=False):
 
     return sentences, spans
 
-def tokenize_sentence(sentence):
 
-    iterator = re.finditer("""
+tokenize_pattern = """
     (                                     # <unk>
       (?:^|(?<=\s))
-        \S*(\S)\\2{3,}\S*                 # char repeated more than 3 times
+        \S*(\S)\\2{{3,}}\S*               # char repeated more than 3 times
+        | [-]{{3}}                        # dash repeated more than 2 times
       (?=\s|$)
+      | :[^\s:@]+?@                       # password in URL
     ) |
     (                                     # <num>
       (?:[-+]?\d+(?:[.,]\d+)*)            # anything numeric looking
       | (?:[.,]\d+)
     ) |
     (                                     # word
-      (?:[-]{0,2}                         # allow command line options
-        [^\W\d]\w*(?:[-'][\w]+)*[']?)     # word, not starting with a digit
+      (?:[-]{{0,2}}                       # allow command line options
+        [^\W\d]\w*(?:[-'´΄][\w]+)*        # word, not starting with a digit
+        [{trailing_characters}'´΄]?)
       | <unk> | <s> | </s> | <num>        # pass through control words
       | <bot:[a-z]*>                      # pass through begin of text merkers
       | (?:^|(?<=\s))
           (?:
-            \|                            # common space delimited operators
+            \| {standalone_operators}     # common space delimited operators
           )
         (?=\s|$)
     )
-    """, sentence, re.UNICODE|re.DOTALL|re.VERBOSE)
+    """
+# Don't learn "-" or "--" as standalone tokens...
+TEXT_PATTERN = re.compile(tokenize_pattern.format(
+                          trailing_characters = "",
+                          standalone_operators = ""),
+                          re.UNICODE|re.DOTALL|re.VERBOSE)
+# ...but recognize them in a prediction context as start of a cmd line option.
+CONTEXT_PATTERN = re.compile(tokenize_pattern.format(
+                          trailing_characters = "-",
+                          standalone_operators = "| [-]{1,2}"),
+                          re.UNICODE|re.DOTALL|re.VERBOSE)
 
+def tokenize_sentence(sentence, is_context = False):
+
+    if is_context:
+        matches = CONTEXT_PATTERN.finditer(sentence)
+    else:
+        matches = TEXT_PATTERN.finditer(sentence)
     tokens = []
     spans = []
 
-    for match in iterator:
+    for match in matches:
         groups = match.groups()
         if groups[3]:
             tokens.append(groups[3])
@@ -284,7 +337,7 @@ def tokenize_sentence(sentence):
 
     return tokens, spans
 
-def tokenize_text(text):
+def tokenize_text(text, is_context = False):
     """ Split text into word tokens.
         The result is ready for use in learn_tokens().
 
@@ -305,7 +358,7 @@ def tokenize_text(text):
     spans = []
     sentences, sentence_spans = split_sentences(text)
     for i, sentence in enumerate(sentences):
-        ts, ss = tokenize_sentence(sentence)
+        ts, ss = tokenize_sentence(sentence, is_context)
 
         sbegin = sentence_spans[i][0]
         ss = [[s[0]+sbegin, s[1]+sbegin] for s in ss]
@@ -323,12 +376,12 @@ def tokenize_context(text):
     """ Split text into word tokens + completion prefix.
         The result is ready for use in predict().
     """
-    tokens, spans = tokenize_text(text)
+    tokens, spans = tokenize_text(text, is_context = True)
     if not re.match("""
-                  ^$                              # empty string
-                | .*[-'\w]$                       # word at the end
-                | (?:^|.*\s)[\+\-\*/=\<>&\^|]=?$  # operator, equal sign
-                | .*(\S)\\1{3,}$                  # anything repeated > 3 times
+                  ^$                             # empty string?
+                | .*[-'´΄\w]$                    # word at the end?
+                | (?:^|.*\s)[|]=?$               # recognized operator?
+                | .*(\S)\\1{3,}$                 # anything repeated > 3 times?
                 """, text, re.UNICODE|re.DOTALL|re.VERBOSE):
         tokens.append("")
         tend = len(text)
