@@ -20,6 +20,7 @@ import tempfile
 import unittest
 from Onboard.pypredict import *
 
+
 class _TestPatterns(unittest.TestCase):
 
     def __init__(self, test, text, result):
@@ -209,8 +210,8 @@ class _TestMultiOrder(unittest.TestCase):
 class _TestModel(unittest.TestCase):
 
     def setUp(self):
-        self.tmp_dir = tempfile.TemporaryDirectory(prefix="test_onboard_")
-        self.dir = self.tmp_dir.name
+        self._tmp_dir = tempfile.TemporaryDirectory(prefix="test_onboard_")
+        self._dir = self._tmp_dir.name
 
     def test_case_insensitive(self):
         model = DynamicModel()
@@ -269,7 +270,7 @@ class _TestModel(unittest.TestCase):
         self.assertEqual(choices, ['ABCDE'])
 
     def test_save_load_unigram_model(self):
-        fn = os.path.join(self.dir, "unigram.lm")
+        fn = os.path.join(self._dir, "unigram.lm")
 
         model = UnigramModel()
         tokens = tokenize_text("ccc bbb uu fff ccc ee")[0]
@@ -311,7 +312,7 @@ class _TestModel(unittest.TestCase):
         )
 
     def test_save_load_trigram_model(self):
-        fn = os.path.join(self.dir, "unigram.lm")
+        fn = os.path.join(self._dir, "unigram.lm")
 
         model = DynamicModel()
         tokens = tokenize_text("ccc bbb uu fff ccc ee")[0]
@@ -366,7 +367,7 @@ class _TestModel(unittest.TestCase):
         )
 
     def test_read_order(self):
-        fn = os.path.join(self.dir, "model.lm")
+        fn = os.path.join(self._dir, "model.lm")
 
         self.assertEqual(read_order(fn), None) # file not found
 
@@ -389,18 +390,6 @@ def suite():
     tests = [
          ["", [], [], []],
          ["abc", ["abc"], ["abc"], ["abc"]],
-         #["-", [], [], [u"-"]],
-         ["1", ["<num>"], ["<num>"], ["1"]],
-         ["123", ['<num>'], ['<num>'], ["123"]],
-         ["-3", ["<num>"], ["<num>"], ["-3"]],
-         ["+4", ["<num>"], ["<num>"], ["+4"]],
-         ["123.456", ["<num>"], ["<num>"], ["123.456"]],
-         ["123,456", ["<num>"], ["<num>"], ["123,456"]],
-         ["100,000.00", ["<num>"], ["<num>"], ["100,000.00"]],
-         ["100.000,00", ["<num>"], ["<num>"], ["100.000,00"]],
-         [".5", ["<num>"], ["<num>"], [".5"]],
-         ["-option --option", ['-option', '--option'], ['-option', '--option'],
-             ['-option --option']],
          ["We saw wha", ['We', 'saw', 'wha'], ['We', 'saw', 'wha'],
              ['We saw wha']],
          ["We saw whales", ['We', 'saw', 'whales'],
@@ -440,10 +429,6 @@ def suite():
          ["sentence.\n sentence. ", ['sentence', '<s>', 'sentence', '<s>'],
              ['sentence', '<s>', 'sentence', '<s>', ''],
              ['sentence.', 'sentence.', '']],
-         ["<bot:txt> sentence. sentence. ", 
-             ['<bot:txt>', 'sentence', '<s>', 'sentence', '<s>'],
-             ['<bot:txt>', 'sentence', '<s>', 'sentence', '<s>', ''],
-             ['<bot:txt> sentence.', 'sentence.', '']],
          ["sentence. \nsentence.", ['sentence', '<s>', 'sentence'],
              ['sentence', '<s>', 'sentence', ''],
              ['sentence.', 'sentence.']],
@@ -454,9 +439,6 @@ def suite():
              ['sentence', 'quote', '<s>', 'sentence', ''],
              ['sentence "quote."', 'sentence.']],
          ["sentence <s>", ['sentence'], ['sentence', ''], ['sentence']],
-         ["<unk> <s> </s> <num>", ['<unk>', '<s>', '</s>', '<num>'],
-             ['<unk>', '<s>', '</s>', '<num>', ''],
-             ['<unk>', '</s> <num>']],
          [""""double quotes" 'single quotes'""",
              ['double', 'quotes', 'single', "quotes'"],
              ['double', 'quotes', 'single', "quotes'"],
@@ -465,14 +447,6 @@ def suite():
              ['parens', 'brackets', 'braces'],
              ['parens', 'brackets', 'braces', ''],
              ['(parens) [brackets] {braces}']],
-         ["repeats: a aa aaa aaaa aaaaa",
-             ['repeats', '<s>', 'a', 'aa', 'aaa', '<unk>', '<unk>'],
-             ['repeats', '<s>', 'a', 'aa', 'aaa', '<unk>', '<unk>'],
-             ['repeats:', 'a aa aaa aaaa aaaaa']],
-         ["www", ['www'], ['www'], ['www']],
-         ["www.", ['www'], ['www', ''], ['www.']],
-         ["www,", ['www'], ['www', ''], ['www,']],
-         ["www ", ['www'], ['www', ''], ['www']],
          ["\nnewline ", ['newline'], ['newline', ''], ['newline']],
          ["double\n\nnewline ", ['double', '<s>', 'newline'],
              ['double', '<s>', 'newline', ''], ['double', 'newline']],
@@ -481,20 +455,72 @@ def suite():
          ["double_newline \n \n \n", ['double_newline', '<s>'],
              ['double_newline', '<s>', ''], ['double_newline', '']],
          ["dash-dash", ["dash-dash"], ["dash-dash"], ["dash-dash"]],
-         ["dash-", ['dash'], ['dash'], ['dash-']],
+         ["dash-", ['dash'], ['dash-'], ['dash-']],
          ["single quote's", ['single', "quote's"], ['single', "quote's"],
              ["single quote's"]],
          ["single quote'", ['single', "quote'"], ['single', "quote'"],
              ["single quote'"]],
          ["under_score's", ["under_score's"], ["under_score's"],
              ["under_score's"]],
+         ["Greek Γ´", ['Greek', 'Γ´'], ['Greek', 'Γ´'], ["Greek Γ´"]], # U+00b4
+         ["Greek Γ΄", ['Greek', 'Γ΄'], ['Greek', 'Γ΄'], ["Greek Γ΄"]], # U+0384
+
+         # command line handling
+         ["-option --option", ['-option', '--option'], ['-option', '--option'],
+             ['-option --option']],
+         ["cmd -", ['cmd'], ['cmd', '-'], ['cmd -']],
+         ["cmd - ", ['cmd'], ['cmd', '-', ''], ['cmd -']],
+         ["cmd --", ['cmd'], ['cmd', '--'], ['cmd --']],
+         ["cmd -- ", ['cmd'], ['cmd', '--', ''], ['cmd --']],
+         ["cmd ---", ['cmd', '<unk>'], ['cmd', '<unk>'], ['cmd ---']],
          ["|", ['|'], ['|'], ['|']],
+         ["find | grep", ['find', '|', 'grep'], ['find', '|', 'grep'],
+             ['find | grep']],
+         ["cat /", ['cat'], ['cat', ''], ['cat /']],
+
+         # passing through control words
+         ["<unk> <s> </s> <num>", ['<unk>', '<s>', '</s>', '<num>'],
+             ['<unk>', '<s>', '</s>', '<num>', ''],
+             ['<unk>', '</s> <num>']],
+
+         # <unk>
+         ["repeats: a aa aaa aaaa aaaaa",
+             ['repeats', '<s>', 'a', 'aa', 'aaa', '<unk>', '<unk>'],
+             ['repeats', '<s>', 'a', 'aa', 'aaa', '<unk>', '<unk>'],
+             ['repeats:', 'a aa aaa aaaa aaaaa']],
+
+         # <num>
+         ["1", ["<num>"], ["<num>"], ["1"]],
+         ["123", ['<num>'], ['<num>'], ["123"]],
+         ["-3", ["<num>"], ["<num>"], ["-3"]],
+         ["+4", ["<num>"], ["<num>"], ["+4"]],
+         ["123.456", ["<num>"], ["<num>"], ["123.456"]],
+         ["123,456", ["<num>"], ["<num>"], ["123,456"]],
+         ["100,000.00", ["<num>"], ["<num>"], ["100,000.00"]],
+         ["100.000,00", ["<num>"], ["<num>"], ["100.000,00"]],
+         [".5", ["<num>"], ["<num>"], [".5"]],
+
+         # begin of text markers
          ["<bot:txt> word", ['<bot:txt>', 'word'], ['<bot:txt>', 'word'],
              ['<bot:txt> word']],
          ["<bot:term> word", ['<bot:term>', 'word'], ['<bot:term>', 'word'],
              ['<bot:term> word']],
          ["<bot:url> word", ['<bot:url>', 'word'], ['<bot:url>', 'word'],
              ['<bot:url> word']],
+         ["<bot:txt> sentence. sentence. ",
+             ['<bot:txt>', 'sentence', '<s>', 'sentence', '<s>'],
+             ['<bot:txt>', 'sentence', '<s>', 'sentence', '<s>', ''],
+             ['<bot:txt> sentence.', 'sentence.', '']],
+
+         # URLs
+         ["www",  ['www'], ['www'],     ['www']],
+         ["www ", ['www'], ['www', ''], ['www']],
+         ["www.", ['www'], ['www', ''], ['www.']],
+         ["www,", ['www'], ['www', ''], ['www,']],
+         ["http://user:pass@www.do-mai_n.nl/path/name.ext",
+             ['http', 'user', '<unk>', 'www', 'do-mai_n', 'nl', 'path', 'name', 'ext'],
+             ['http', 'user', '<unk>', 'www', 'do-mai_n', 'nl', 'path', 'name', 'ext'],
+             ['http://user:pass@www.do-mai_n.nl/path/name.ext']],
         ]
 
     # Low-level regex pattern tests
