@@ -24,10 +24,9 @@ from Onboard.LayoutLoaderSVG import LayoutLoaderSVG
 from Onboard.SnippetView     import SnippetView
 from Onboard.Appearance      import Theme, ColorScheme
 from Onboard.Scanner         import ScanMode, ScanDevice
-from Onboard.XInput          import XIDeviceManager, XIEventType, XIEventMask
+from Onboard.XInput          import XIDeviceManager, XIEventType
 from Onboard.utils           import show_ask_string_dialog, \
                                     show_confirmation_dialog, \
-                                    exists_in_path, \
                                     unicode_str, open_utf8
 
 app = "onboard"
@@ -101,17 +100,34 @@ class DialogBuilder(object):
             callback(value)
 
     # checkbox
-    def bind_check(self, name, config_object, key, widget_callback = None):
+    def bind_check(self, name, config_object, key,
+                   config_get_callback = None, config_set_callback = None):
         w = self.wid(name)
-        w.set_active(getattr(config_object, key))
-        if widget_callback:
-            w.connect("toggled", widget_callback, config_object, key)
+        if config_get_callback:
+            active = config_get_callback(config_object, key)
         else:
-            w.connect("toggled", self.bind_check_callback, config_object, key)
-        getattr(config_object, key + '_notify_add')(w.set_active)
+            active = getattr(config_object, key)
+        w.set_active(active)
+        w.connect("toggled", self._bind_check_callback,
+                  config_object, key, config_set_callback)
+        getattr(config_object, key + '_notify_add')( \
+             lambda x: self._notify_check_callback(w, config_object, key,
+                                                     config_get_callback))
 
-    def bind_check_callback(self, widget, config_object, key):
-        setattr(config_object, key, widget.get_active())
+    def _notify_check_callback(self, widget, config_object,
+                              key, config_get_callback):
+        if config_get_callback:
+            active = config_get_callback(config_object, key)
+        else:
+            active = getattr(config_object, key)
+        widget.set_active(active)
+
+    def _bind_check_callback(self, widget, config_object,
+                            key, config_set_callback):
+        if config_set_callback:
+            config_set_callback(config_object, key, widget.get_active())
+        else:
+            setattr(config_object, key, widget.get_active())
 
     # combobox with id column
     def bind_combobox_id(self, name, config_object, key,
@@ -146,8 +162,34 @@ class DialogBuilder(object):
             if not id is None:
                 setattr(config_object, key, int(widget.get_active_id()))
 
+    def select_tree_view_row(self, view, column, value, _it = None):
+        model = view.get_model()
+        if _it is None:
+            _it = model.get_iter_first()
+        while _it:
+            child_it = model.iter_children(_it)
+            if child_it:
+                self.select_tree_view_row(view, column, value, child_it)
+
+            fn = model.get_value(_it, column)
+            if fn == value:
+                sel = view.get_selection()
+                if sel:
+                    sel.select_iter(_it)
+
+            _it = model.iter_next(_it)
+
 
 class Settings(DialogBuilder):
+
+    # column ids of the layout view
+    LAYOUT_COL_NAME = 0
+    LAYOUT_COL_SUMMARY = 1
+    LAYOUT_COL_FILENAME = 2
+    LAYOUT_COL_HAS_ABOUT_INFO = 3
+    LAYOUT_COL_IS_ROW_SENSITIVE = 4
+
+
     def __init__(self,mainwin):
         self.themes = {}       # cache of theme objects
 
@@ -302,12 +344,12 @@ class Settings(DialogBuilder):
                              builder.get_object("docking_enabled_toggle")
         self.docking_box = builder.get_object("docking_box")
 
-        def on_docking_enabled_toggle(widget, config_object, key):
-            setattr(config_object, key, widget.get_active())
+        def on_docking_enabled_config_set(config_object, key, value):
+            setattr(config_object, key, value)
             self.update_window_widgets()
         self.bind_check("docking_enabled_toggle",
                         config.window, "docking_enabled",
-                        widget_callback = on_docking_enabled_toggle)
+                        config_set_callback = on_docking_enabled_config_set)
         self.bind_button("docking_settings_button",
                          lambda widget: DockingDialog().run(self.window))
 
@@ -315,15 +357,9 @@ class Settings(DialogBuilder):
 
         # layout view
         self.layout_view = builder.get_object("layout_view")
-        self.layout_view.append_column( \
-                Gtk.TreeViewColumn(None, Gtk.CellRendererText(), markup=0))
-
         self.user_layout_root = config.get_user_layout_dir()
-
-        self.layout_remove_button = \
-                             builder.get_object("layout_remove_button")
-
-        self.update_layoutList()
+        config.layout_notify_add(lambda x: self.update_layout_view_selection())
+        self.update_layout_view()
         self.update_layout_widgets()
 
         # theme view
@@ -530,6 +566,32 @@ class Settings(DialogBuilder):
     def on_inactive_transparency_delay_changed(self, widget):
         config.window.inactive_transparency_delay = widget.get_value()
 
+    def on_layout_new_button_clicked(self, widget):
+        name = self.get_selected_layout_value(self.LAYOUT_COL_NAME)
+        filename = self.get_selected_layout_filename()
+        if filename:
+            new_layout_name = show_ask_string_dialog(
+                _format("Copy layout '{}' to this new name:", name), self.window)
+            if new_layout_name:
+                new_filename = \
+                    os.path.join(self.user_layout_root, new_layout_name) + \
+                    config.LAYOUT_FILE_EXTENSION
+                LayoutLoaderSVG.copy_layout(filename, new_filename)
+                self.update_layout_view()
+                self.open_user_layout_dir()
+
+    def on_layout_remove_button_clicked(self, event):
+        name = self.get_selected_layout_value(self.LAYOUT_COL_NAME)
+        filename = self.get_selected_layout_filename()
+        if filename:
+            question = _format("Delete layout '{}'?", name)
+            if show_confirmation_dialog(question, self.window):
+                LayoutLoaderSVG.remove_layout(filename)
+
+                config.layout_filename = self.layout_view_model[0][1] \
+                                        if len(self.layout_view_model) else ""
+                self.update_layout_view()
+
     def open_user_layout_dir(self):
         if os.path.exists('/usr/bin/nautilus'):
             os.system(("nautilus --no-desktop %s" %self.user_layout_root))
@@ -541,15 +603,14 @@ class Settings(DialogBuilder):
     def on_layout_folder_button_clicked(self, widget):
         self.open_user_layout_dir()
 
-    def on_personalise_button_clicked(self, widget):
-        new_layout_name = show_ask_string_dialog(
-            _("Enter name for personalised layout"), self.window)
-        if new_layout_name:
-            new_filename = os.path.join(self.user_layout_root, new_layout_name) + \
-                           config.LAYOUT_FILE_EXTENSION
-            LayoutLoaderSVG.copy_layout(config.layout_filename, new_filename)
-            self.update_layoutList()
-            self.open_user_layout_dir()
+    def update_layout_widgets(self):
+        filename = self.get_selected_layout_filename()
+        self.wid("layout_new_button").set_sensitive(not filename is None)
+        self.wid("layout_remove_button").set_sensitive(not filename is None and \
+                                         os.access(filename, os.W_OK))
+        has_about_info = bool( \
+            self.get_selected_layout_value(self.LAYOUT_COL_HAS_ABOUT_INFO))
+        self.wid("layout_about_button").set_sensitive(has_about_info)
 
     def on_scanner_enabled_toggled(self, widget):
         config.scanner.enabled = widget.get_active()
@@ -611,104 +672,122 @@ class Settings(DialogBuilder):
         self.window.destroy()
         Gtk.main_quit()
 
-    def update_layoutList(self):
-        self.layoutList = Gtk.ListStore(str, str)
-        self.layout_view.set_model(self.layoutList)
+    def update_layout_view(self):
+        model = self.layout_view.get_model()
+        self.layout_view_model = model
+        model.clear()
 
-        self.update_layouts(os.path.join(config.install_dir, "layouts"))
-        self.update_layouts(self.user_layout_root)
+        sort_order = ["Small", "Compact", "Full Keyboard", "Phone", "Grid"]
+        layout_infos = self._read_layouts(
+            os.path.join(config.install_dir, "layouts"), sort_order)
 
-    def cb_selected_layout_changed(self):
-        self.update_layouts(self.user_layout_root)
+        system = [li for li in layout_infos
+                    if li.layout_section == "system"]
+        contributed = [li for li in layout_infos
+                    if li.layout_section == "contributions"]
 
-    def on_add_button_clicked(self, event):
-        chooser = Gtk.FileChooserDialog(title=_("Add Layout"),
-                                        parent=self.window,
-                                        action=Gtk.FileChooserAction.OPEN,
-                                        buttons=(Gtk.STOCK_CANCEL,
-                                                 Gtk.ResponseType.CANCEL,
-                                                 Gtk.STOCK_OPEN,
-                                                 Gtk.ResponseType.OK))
-        filterer = Gtk.FileFilter()
-        filterer.add_pattern("*.sok")
-        filterer.add_pattern("*" + config.LAYOUT_FILE_EXTENSION)
-        filterer.set_name(_("Onboard layout files"))
-        chooser.add_filter(filterer)
+        user = self._read_layouts(self.user_layout_root)
 
-        filterer = Gtk.FileFilter()
-        filterer.add_pattern("*")
-        filterer.set_name(_("All files"))
-        chooser.add_filter(filterer)
+        self._add_layout_section(model, system, _("Core layouts"))
+        self._add_layout_section(model, contributed, _("Contributions"))
+        self._add_layout_section(model, user, _("User layouts"))
 
-        response = chooser.run()
-        if response == Gtk.ResponseType.OK:
-            filename = chooser.get_filename()
+        self.layout_view.expand_all()
+        self.update_layout_view_selection()
 
-            f = open_utf8(filename)
-            sokdoc = minidom.parse(f).documentElement
-            for p in sokdoc.getElementsByTagName("pane"):
-                fn = p.attributes['filename'].value
+    def update_layout_view_selection(self):
+        self.select_tree_view_row(self.layout_view, self.LAYOUT_COL_FILENAME,
+                                  config.layout_filename)
 
-                shutil.copyfile("%s/%s" % (os.path.dirname(filename), fn),
-                                "%s%s" % (self.user_layout_root, fn))
+    def _add_layout_section(self, model, lis, section_name):
+        if lis:
+            parent_iter = model.append(None)
+            model.set(parent_iter,
+                    self.LAYOUT_COL_NAME, "<b>{}</b>".format(section_name))
+            for li in lis:
+                child_iter = model.append(parent_iter)
+                model.set(child_iter,
+                        self.LAYOUT_COL_NAME, li.id_string,
+                        self.LAYOUT_COL_SUMMARY, li.summary,
+                        self.LAYOUT_COL_FILENAME, li.filename,
+                        self.LAYOUT_COL_HAS_ABOUT_INFO, li.has_about_info,
+                        self.LAYOUT_COL_IS_ROW_SENSITIVE, bool(li.filename))
 
-            shutil.copyfile(filename,"%s%s" % (self.user_layout_root,
-                                               os.path.basename(filename)))
+    def on_layout_about_button_clicked(self, event = None):
+        fn = self.get_selected_layout_filename()
+        li = self._read_layout(fn)
+        if li is None:
+            return
 
-            self.update_layoutList()
-        chooser.destroy()
+        markup = "<big>{}</big>\n".format(li.id)
+        body = li.description or li.summary
+        if body:
+            markup += "\n{}\n".format(body)
+        if li.author:
+            markup += ("\n" + _("Author: {}") + "\n").format(li.author)
+        markup += "\n <tt>{}</tt>\n".format(li.filename)
 
-    def on_layout_remove_button_clicked(self, event):
-        sel = self.layout_view.get_selection()
-        if sel:
-            filename = self.layoutList.get_value(sel.get_selected()[1], 1)
+        dialog = Gtk.MessageDialog(title=_("About Layout"),
+                                   message_type=Gtk.MessageType.QUESTION,
+                                   buttons=Gtk.ButtonsType.OK)
+        dialog.set_markup(markup)
+        dialog.set_transient_for(self.window)
+        dialog.run()
+        dialog.destroy()
 
-            LayoutLoaderSVG.remove_layout(filename)
+    def _read_layouts(self, path,  sort_order=()):
+        filenames = self._find_layouts(path)
 
-            config.layout_filename = self.layoutList[0][1] \
-                                     if len(self.layoutList) else ""
-        self.update_layoutList()
-
-    def update_layouts(self, path):
-
-        filenames = self.find_layouts(path)
-
-        layouts = []
+        layout_infos = []
         for filename in filenames:
+            li = self._read_layout(filename, sort_order)
+            if li:
+                layout_infos.append(li)
+
+        return sorted(layout_infos, key=lambda x: x.sort_key)
+
+    def _read_layout(self, filename, sort_order=()):
+        li = None
+
+        if filename and os.path.exists(filename):
             file_object = open_utf8(filename)
             try:
-                sokdoc = minidom.parse(file_object).documentElement
+                dom_node = minidom.parse(file_object).documentElement
 
-                value = sokdoc.attributes["id"].value
-                if os.access(filename, os.W_OK):
-                    layouts.append((value.lower(), value, filename))
-                else:
-                    layouts.append((value.lower(),
-                                   "<i>{0}</i>".format(value),
-                                   filename))
+                class LayoutInfo: pass
+                li = LayoutInfo()
+                id = dom_node.attributes["id"].value
+                sort_priority = sort_order.index(id) \
+                                if id in sort_order else 1000
+                li.id = id
+                li.filename = filename
+                li.layout_section = self._get_dom_string(dom_node, "section")
+                li.summary = _(self._get_dom_string(dom_node, "summary"))
+                li.description = \
+                    _(self._get_dom_string(dom_node, "description"))
+                li.author = self._get_dom_string(dom_node, "author")
+                li.sort_key = (li.layout_section.lower(), sort_priority, id.lower())
+                li.has_about_info = True
+                li.id_string = id
 
-            except ExpatError as xxx_todo_changeme:
-                (strerror) = xxx_todo_changeme
-                print("XML in %s %s" % (filename, strerror))
-            except KeyError as xxx_todo_changeme1:
-                (strerror) = xxx_todo_changeme1
-                print("key %s required in %s" % (strerror,filename))
+            except ExpatError as ex:
+                _logger.error("XML in %s %s" % (filename, unicode_str(ex)))
+                li = None
+            except KeyError as ex:
+                _logger.error("key %s required in %s" % (unicode_str(ex), filename))
+                li = None
 
             file_object.close()
 
-        for key, value, filename in sorted(layouts):
-            it = self.layoutList.append((value, filename))
-            if filename == config.layout_filename:
-                sel = self.layout_view.get_selection()
-                if sel:
-                    sel.select_iter(it)
+        return li
 
-    def update_layout_widgets(self):
-        filename = self.get_selected_layout_filename()
-        self.layout_remove_button.set_sensitive(not filename is None and \
-                                         os.access(filename, os.W_OK))
+    @staticmethod
+    def _get_dom_string(dom_node, attribute, default = ""):
+        if dom_node.hasAttribute(attribute):
+            return dom_node.attributes[attribute].value
+        return default
 
-    def find_layouts(self, path):
+    def _find_layouts(self, path):
         layouts = []
         try:
             files = os.listdir(path)
@@ -720,6 +799,14 @@ class Settings(DialogBuilder):
                 layouts.append(os.path.join(path, filename))
         return layouts
 
+    def on_layout_view_select_cursor_row(self, treeview, *args):
+        print(args)
+        return False
+
+    def on_layout_view_row_activated(self, treeview, path, view_column):
+        #self.on_layout_about_button_clicked() # too annoying
+        pass
+
     def on_layout_view_cursor_changed(self, widget):
         filename = self.get_selected_layout_filename()
         if filename:
@@ -731,7 +818,16 @@ class Settings(DialogBuilder):
         if sel:
             it = sel.get_selected()[1]
             if it:
-                return self.layoutList.get_value(it,1)
+                return self.layout_view_model.get_value(it,
+                                                    self.LAYOUT_COL_FILENAME)
+        return None
+
+    def get_selected_layout_value(self, col_index):
+        sel = self.layout_view.get_selection()
+        if sel:
+            it = sel.get_selected()[1]
+            if it:
+                return self.layout_view_model.get_value(it, col_index)
         return None
 
     def on_new_theme_button_clicked(self, widget):
@@ -931,8 +1027,12 @@ class PageWordSuggestions(DialogBuilder):
                 .connect_after("toggled", lambda x: self._update_ui())
         self.bind_check("enable_word_suggestions_toggle",
                         config.word_suggestions, "enabled")
+
+        self.wid("auto_learn_toggle") \
+                .connect_after("toggled", lambda x: self._update_ui())
         self.bind_check("auto_learn_toggle",
                             config.wp, "auto_learn")
+
         self.bind_check("punctuation_assistence_toggle",
                             config.wp, "punctuation_assistance")
         self.bind_check("auto_capitalization_toggle",
@@ -941,6 +1041,21 @@ class PageWordSuggestions(DialogBuilder):
                             config.typing_assistance, "auto_correction")
         self.bind_check("enable_spell_check_toggle",
                             config.word_suggestions, "spelling_suggestions_enabled")
+
+        self.bind_check("language_button_toggle",
+            config.word_suggestions, "wordlist_buttons",
+            config_get_callback = lambda co, key: \
+                    config.word_suggestions.can_show_language_button(),
+            config_set_callback = lambda co, key, value: \
+                    config.word_suggestions.show_language_button(value))
+
+        self.bind_check("pause_learning_button_toggle",
+            config.word_suggestions, "wordlist_buttons",
+            config_get_callback = lambda co, key: \
+                    config.word_suggestions.can_show_pause_learning_button(),
+            config_set_callback = lambda co, key, value: \
+                    config.word_suggestions.show_pause_learning_button(value))
+
         #self.bind_check("show_context_line_toggle",
         #                config.word_suggestions, "show_context_line")
         self._init_spell_checker_backend_combo()
@@ -962,6 +1077,8 @@ class PageWordSuggestions(DialogBuilder):
     def _update_ui(self):
         self.wid("word_suggestions_general_box1") \
                 .set_sensitive(config.are_word_suggestions_enabled())
+        self.wid("pause_learning_button_toggle") \
+                .set_sensitive(config.word_suggestions.auto_learn)
 
 
 class DockingDialog(DialogBuilder):
